/*  ============================================================================
 *   Copyright (c) Texas Instruments Inc 2002, 2003, 2004, 2005, 2006, 2008
 *
 *   Use of this software is controlled by the terms and conditions found in the
 *   license agreement under which this software has been supplied.
 *   ===========================================================================
 */

/* ============================================================================
 *   @file  Emac_example.c
 *
 *   PATH:  \$(CSLPATH)\\example\\emac\\src
 *
 *   @brief  Example for EMAC CSL
 *
 *  ============================================================================
 *   @n <b> Example Description </b>
 *   @n This example, is a simple loop back configured for channel 0. If emac is configured
 *      no need to configure ECTL, EMAC src configures it. Here the EMAC src configures 
 *      interrupts for one Rx channel and one Tx channel.
 *      This example tested for 100 packets.
 *
 *      LOCAL_LOOPBACK performs loopback in the MAC, however since this example
 *      also does not check for a LINK on the physical PHY as it does not configire MDIO.
 *
 * =============================================================================
 *
 *   <b> Procedure to run the example </b>
 *   @verbatim
 *      1. Configure the CCS setup to work with.
 *      2. Launch CCS window.
 *      3. Open project Emac_local_loopback_example.pjt
 *      4. Build the project and load the .out file of the project.
 *      5. Run the example.
 *   @endverbatim
 *
 */

#include <stdio.h>
#include <csl_emac.h>

#define LOCAL_LOOPBACK  1       /* Loopback internal to the EMAC */
#define PHY_LOOPBACK    0       /* Loopback at the PHY device    */

/*
 * Raw Data Buffers
 *
 * This example uses internal memory, so we could just use 1518 as our
 * buffer size. However, if the buffers were in cacheable external
 * memory, we'd have to make sure each was cache aligned, and
 * thus use a size of at least 1536 because a buffer must fill an
 * the entire cache line.
 */
#define PKT_MAX 32
EMAC_Pkt packet_header[PKT_MAX];
Uint8 packet_buffer[PKT_MAX][1536];
Uint8 packet_sourcedata[1536];


/*
 * Packet Queue
 *
 * This simple queue object is just for our example packet
 * queue. It is not required for using EMAC/MDIO
 *
 */
typedef struct app_pktq {
  Uint32            Count;      /**< Number of packets in queue */
  EMAC_Pkt          *pHead;     /**< Pointer to first packet    */
  EMAC_Pkt          *pTail;     /**< Pointer to last packet     */
} APP_PKTQ;

/*
 * Queue Helper Functions
 */
static EMAC_Pkt *App_pqPop( APP_PKTQ *pq );
static void App_pqPush( APP_PKTQ *pq, EMAC_Pkt *pPktHdr );

/*
 * Declare our packet queues
 */
APP_PKTQ     FreeQueue;             /* Free packets for RX or TX */
APP_PKTQ     RxQueue;               /* Received packets */

/*
 * Declare some local status variables
 */
Handle            hEMAC = 0;       /* Handle to our EMAC instance */
volatile Uint32   LinkStatus = 0;  /* Our current link status */
volatile Uint32   LocalTicks = 0;  /* Current time in 100ms ticks */


/* MAC Rx interrupt is HWINT6, 
   MAC Tx interrupt is HWINT7
   0xC0 is for MAC RX & Tx interrupts */

extern cregister volatile unsigned int IER;
#define OUREMAC_enter() IER &= ~0xC0
#define OUREMAC_exit()  IER |= 0xC0


/*
 * Declare some local functions
 */
static EMAC_Pkt *GetPacket(Handle hApplication);
static void     FreePacket(Handle hApplication, EMAC_Pkt *pPKT);
static EMAC_Pkt *RxPacket( Handle hApplication, EMAC_Pkt *pPKT);
static void     StatusUpdate(Handle hApplication);
static void     StatisticsUpdate(Handle hApplication);
static int      verify_packet( EMAC_Pkt *pPkt, Uint32 size );

static EMAC_Status status;
static char *LinkStr[] ={ 
						"No Link", 
						"10Mb/s Half Duplex", 
						"10Mb/s Full Duplex",
						"100Mb/s Half Duplex", 
						"100Mb/s Full Duplex", 
						"1000Mb/s Full Duplex"
						};


/*
 * Declare some external functions
 */
extern void DoTimerInitialization(void);
extern void DoInterruptsInitialization(void);
extern void DoInterruptEnd(void);


/*
 * =============================================================================
 *   @func   Main 
 *
 *   @desc
 *      Main Example Entry Point.  
 *      
 *   @arg
 *      None
 *      
 *   @expected result
 *      If the example passes, it displays the message "Example Passed"
 *      If the example fails, it displays the messae "Example Failed"
 *
 *   @eg
 *      exampleMain(); // Read description.
 * =============================================================================
 */
void main()
{
    EMAC_Config ecfg;
	EMAC_AddrConfig *addrCfg;    
    Uint32      i,size, j;
    Uint32      TxCount=0;
	Uint32		RxCount=0;
	Uint32		count = 100;//(1514-60)+1;
    EMAC_Pkt    *pPkt;

    /*
     * Basic Initialization
     */                    
    printf("EMAC example\n");

    /* Init packet data array */
    for( i=0; i<1514; i++ ) {
		packet_sourcedata[i] = i;
	}

    /* Initialize our buffer pool */
    memset( &FreeQueue, 0, sizeof(APP_PKTQ) );
    memset( &RxQueue, 0, sizeof(APP_PKTQ) );
    for( i=0; i<PKT_MAX; i++ ) {
        /*
         * Init the buffer headers. Note that the buffer pointer
         * and buffer length are fixed and to not change
         */
        memset( &packet_header[i], 0, sizeof(EMAC_Pkt) );
        packet_header[i].pDataBuffer = packet_buffer[i];
        packet_header[i].BufferLen   = 1518;
        App_pqPush( &FreeQueue, &packet_header[i] );
    }

    /*
     * Initialize Configuration and Open EMAC
     */
    
    /* Not using the MDIO configuration */
	ecfg.UseMdio = 0;
	/* packet size */
	ecfg.PktMTU = 1518;
	/* Total 9 MAC addresses allocated for receive channel assigned */
	ecfg.TotalNumOfMacAddrs = 9;
	/* selects CPPI RAM for Descriptor memory */
	ecfg.DescBase = EMAC_DESC_BASE_CPPI;

    /* Setup the EMAC configuration */
#if LOCAL_LOOPBACK
    ecfg.ModeFlags      = EMAC_CONFIG_MODEFLG_MACLOOPBACK | EMAC_CONFIG_MODEFLG_FULLDUPLEX | EMAC_CONFIG_MODEFLG_GIGABIT;
#else
    ecfg.ModeFlags      = 0;
#endif

#if PHY_LOOPBACK
    ecfg.MdioModeFlags  = MDIO_MODEFLG_FD1000 | MDIO_MODEFLG_LOOPBACK;
#else
    ecfg.MdioModeFlags  = MDIO_MODEFLG_FD1000;
#endif

	/* channel usage */
    ecfg.ChannelInfo.TxChanEnable = 1;  /* 00000001, channel 0 */

	ecfg.ChannelInfo.RxChanEnable = 1;  /* 00000001, channel 0 */

    ecfg.RxMaxPktPool   = 8;
    ecfg.pfcbGetPacket  = &GetPacket;
    ecfg.pfcbFreePacket = &FreePacket;
    ecfg.pfcbRxPacket   = &RxPacket;
    ecfg.pfcbStatus     = &StatusUpdate;
    ecfg.pfcbStatistics = &StatisticsUpdate;

	/*	Configures the number of MAC addresses per channel 
	 *		Hardware gives support for 32 MAC addresses for 8 receive channels
	 *	    Here total 9 MAC addresses are assigned to receive channel 0 
	 *		MAC addresses and channels allocated are like mentioned below
	 *			core no	   channel assigned       MAC address
	 *			core0         channel 0        	00.01.02.03.04.05
	 *											10.11.12.13.14.15
	 *											20.21.22.23.24.25
	 *											30.31.32.33.34.35
	 *											40.41.42.43.44.45
	 *											50.51.52.53.54.55
	 *											60.61.62.63.64.65
	 *											70.71.72.73.74.75
	 *											80.81.82.83.84.85(address used for loopback test)
	 */
	ecfg.MacAddr = (EMAC_AddrConfig **)
     		       malloc(ecfg.TotalNumOfMacAddrs * sizeof(EMAC_AddrConfig *));

	for (j=0; j<ecfg.TotalNumOfMacAddrs; j++){
		ecfg.MacAddr[j] = (EMAC_AddrConfig *)malloc(sizeof(EMAC_AddrConfig));
	}
     		       
    for(j=0; (Uint8)j<(ecfg.TotalNumOfMacAddrs); j++){
		addrCfg = ecfg.MacAddr[j];
   		for (i=0; i<6; i++){
		 	addrCfg->Addr[i] = j * 0x10 + i;
		}
	}

	addrCfg = ecfg.MacAddr[8];
	for( i=0; i<6; i++ ) {
		packet_sourcedata[i] = addrCfg->Addr[i];
	}

    /* Open EMAC instance */
    printf("Calling EMAC_open()\n");
    i = EMAC_open( 1, (Handle)0x12345678, &ecfg, &hEMAC );
    if(i)
    {
        printf("Returned error %08x\n",i);
        goto exit;
    }

    /* Set the receive filter */
    /* Use our re-entrancy gate to call EMAC functions */
    printf("Calling EMAC_setReceiveFilter()\n");
    OUREMAC_enter();
    i = EMAC_setReceiveFilter(hEMAC, EMAC_RXFILTER_DIRECT);
    OUREMAC_exit();
    if(i)
    {
        printf("Returned error %08x\n",i);
        goto close_emac;
    }

    /* Do timer initialization */
    DoTimerInitialization();

    /* Do interrupt setup */
	DoInterruptsInitialization();

    /*
     * Wait for the device to link
     *
     * The device can not send or receive packets until its
     * linked.
     *
     */
    printf("Waiting for link...\n");
    LocalTicks = 0;
	if(ecfg.UseMdio){
        while( !LinkStatus && LocalTicks < 10000);
   }

    /* Need to print this here, as it's not possible from the callback function */
    EMAC_getStatus( hEMAC, &status);

    LinkStatus = status.MdioLinkStatus;
    if( LinkStatus <= 5 )
        printf("Link Status  : %s on PHY number %d\n",
               LinkStr[LinkStatus],status.PhyDev);

    printf("Packets Held : %d-RX  %d-TX\n",
           status.RxPktHeld, status.TxPktHeld);
    if( status.FatalError )
        printf("Fatal Error  : %d\n",status.FatalError);

    if( !LinkStatus )
        printf("\nWARNING: No device LINK.\n");

   /* Start the example */
    printf("\nStarting packet transfer...\n");

    LocalTicks = 0;
    while( RxCount<count ) {
        /* Check for timeout */
	  	    if(LocalTicks > 100) {
	   	        printf("ERROR: Timeout waiting for TX/RX\n");
        	    break;
		}

        /* See if we should send a packet */
        if( TxCount<count && TxCount<(RxCount+8) ) {
            /*
             * Send a packet
            */

            /* First get a packet buffer from our private queue */
            pPkt = App_pqPop( &FreeQueue );
            if( pPkt ) {
                /* Fill buffer with a pattern, using "count+60" for size */
                size = TxCount + 60;
                memcpy( pPkt->pDataBuffer, packet_sourcedata, size);

                /* Send the packet */
                pPkt->Flags      = EMAC_PKT_FLAGS_SOP | EMAC_PKT_FLAGS_EOP;
                pPkt->ValidLen   = size;
                pPkt->DataOffset = 0;
                pPkt->PktChannel = 0;
                pPkt->PktLength  = size;
                pPkt->PktFrags   = 1;

                /*
                 * IMPORTANT: If our data packet were in EXTERNAL memory,
                 * we would have to clean it from the CACHE before submitting
                 * it to the EMAC module!!!
                 *
                 * We should probably do the clean operation right here.
                 */

                /* Use our re-entrancy gate to call EMAC functions */
                OUREMAC_enter();
                i = EMAC_sendPacket( hEMAC, pPkt );
                OUREMAC_exit();
                if(i) {
                    printf("EMAC_sendPacket() returned error %08x\n",i);
                    break;
                }

                /* Bump the TX count */
                TxCount++;
            }
        }

        /*
         * See if we received a packet
         */
        if( pPkt = App_pqPop( &RxQueue ) ) {
            /* Validate this packet */
            if( !verify_packet( pPkt, RxCount + 60 ) ) {
                printf("ERROR: Verify failure on packet %d\n",RxCount);
                break;
            }

            /* Free the packet */
            App_pqPush( &FreeQueue, pPkt );

            /* Bump the RX count */
            RxCount++;
        }
    }

    printf("Sent %d of %d packets, received %d packets\n",
            TxCount, count, RxCount);

    /*
     * Close EMAC
     */
close_emac:

    /* Since we're done, we leave the interrupt disabled */
	DoInterruptEnd();
  
    printf("Calling EMAC_close()\n");
    i = EMAC_close( hEMAC );
    hEMAC = 0;
    if(i) {
        printf("Returned error %08x\n",i);
        goto exit;
    }

exit:
    if( TxCount == count && RxCount == count )
        printf("\nExample Passed\n");
    else {
        printf("\nExample Failed\n");
        printf("\nMake sure to use a loopback connector if not\n");
        printf("using local loopback (#define LOCAL_LOOPBACK 1)\n");
    }

    exit (0);
}


/*
 * =============================================================================
 *   @func   HwRxInt
 *
 *   @desc
 *      Interrupt service routine called as a result of the EMAC hardware Rx
 *      interrupt.
 *
 *   @arg
 *      None
 *      
 *   @eg
 *      HwRxInt(); // Refer description
 * =============================================================================
 */
void interrupt HwRxInt(void)
{
    Uint32 i;

    if (hEMAC)
    {
        i = EMAC_RxServiceCheck(hEMAC);
        if (i) 
        {
            return;
         	// printf("EMAC_RxServiceCheck() returned error %08x\n",i);
        }
    }
    return;
}


/*
 * =============================================================================
 *   @func   HwTxInt
 *
 *   @desc
 *      Interrupt service routine called as a result of the EMAC hardware Tx
 *      interrupt.
 *
 *   @arg
 *      None
 *      
 *   @eg
 *      HwTxInt(); // Refer description
 * =============================================================================
 */
void interrupt HwTxInt(void)
{
    Uint32 i;

    if (hEMAC) 
    {
        i = EMAC_TxServiceCheck(hEMAC);
        if (i) 
        {
            return;
         	// printf("EMAC_TxServiceCheck() returned error %08x\n",i);
        }
    }
    return;
}


/*
 * =============================================================================
 *   @func   TimerTick
 *
 *   @desc
 *      Timer tick service routine - called from 100mS PRD
 *
 *   @arg
 *      None
 *      
 *   @eg
 *      TimerTick(); // Refer description
 * =============================================================================
 */
void interrupt TimerTick(void)
{
   /* Keep track of time for our local timeout loops */

    LocalTicks++;

    /* Poll the EMAC */
    if (hEMAC) 
    {
        OUREMAC_enter();
        EMAC_timerTick(hEMAC);
        OUREMAC_exit();
    }
}
/*
 * =============================================================================
 *   @func   GetPacket - Get empty packet for RX.
 *
 *   @desc
 *      This function is called from the EMAC module to get an empty packet 
 *      buffer. We need to return a packet buffer to the EMAC or return NULL
 *      if there are no buffers available.
 *
 *   @arg
 *      hApplication    Handle to the application
 *      
 *   @eg
 *      Handle  hApp;
 *      ... // Initialize and get a value to handle- done elsewhere.
 *      GetPacket(hApp); // Refer description
 *      ...
 * =============================================================================
 */
static EMAC_Pkt *GetPacket( Handle hApplication )
{
    EMAC_Pkt *pPkt;

    /* Verify our handle came back OK. We don't use it in this example */
    if( (Uint32)hApplication != 0x12345678 ) {
        //printf("GetPacket(): Bad App Handle!\n");
        return(0);
    }

    /* Pop a packet off our local free queue */
    pPkt = App_pqPop(&FreeQueue);

    if( pPkt ) {
        /*
         * Here we tell the EMAC what offset to use by setting
         * the DataOffset field in the packet.
         */
        pPkt->DataOffset = 0;

        /*
         * IMPORTANT: If our data packet were in EXTERNAL memory,
         * we would have to invalidated it from the CACHE before
         * returning it to the EMAC module!!!
         *
         * We should probably do the clean operation right here.
         * However, we are most likely inside an ISR right now.
         * It may be better to invalidate the buffer before it
         * is pushed on the FreeQueue.
         */
    }
    return( pPkt );
}
/*
 * =============================================================================
 *   @func   FreePacket
 *
 *   @desc
 *      Free packet that originated from TX or GetPacket(). This function is
 *      called from the EMAC module to free a packet buffer after a TX 
 *      operation (or RX in the case of a RX shutdown).
 *
 *   @arg
 *      hApplication    handle to the application
 *      pPKT            pointer to the EMAC packet
 *      
 *   @eg
 *      Handle   hApp;
 *      EMAC_Pkt *emacPkt;
 *      ...
 *      FreePacket(hApp, emacPkt); // Refer description
 *      ...
 * =============================================================================
 */
static void FreePacket( Handle hApplication, EMAC_Pkt *pPKT )
{
    /* Verify our handle came back OK. We don't use it in this example */
    if( (Uint32)hApplication != 0x12345678 ) {
        return;
    }
    App_pqPush( &FreeQueue, pPKT );
}

/*
 * 
 *
 */
/*
 * =============================================================================
 *   @func   RxPacket - Reveived packet from the Network.
 *
 *   @desc
 *      This function is called by the EMAC to indicate a receive. Here
 *      we just push the packet onto our receive queue.
 *      
 *      This function returns a free packet to replace the RX packet on
 *      the EMAC queue. If there are no free packets available, we
 *      return NULL.
 *
 *   @arg
 *      hApplication    handle to the application
 *      pPKT            pointer to the EMAC packet
 *      
 *   @eg
 *      Handle   hApp;
 *      EMAC_Pkt *emacPkt;
 *      ...
 *      RxPacket(hApp, emacPkt); // Refer description
 *      ...
 * =============================================================================
 */
static EMAC_Pkt *RxPacket( Handle hApplication, EMAC_Pkt *pPKT )
{
    /* Verify our handle came back OK. We don't use it in this example */
    if( (Uint32)hApplication != 0x12345678 ) {
        return(0);
    }

    /* Push this packet onto our local receive queue */
    App_pqPush( &RxQueue, pPKT );

    /* We must return a free packet to replace the one we were given */
    return( GetPacket(hApplication) );
}

/*
 * =============================================================================
 *   @func   StatusUpdate
 *
 *   @desc
 *      The EMAC or MDIO status has changed 
 *
 *   @arg
 *      hApplication    handle to the application
 *      
 *   @eg
 *      Handle   hApp;
 *      ...
 *      StatusUpdate(hApp); // Refer description
 *      ...
 * =============================================================================
 */
static void StatusUpdate( Handle hApplication )
{

    /* Verify our handle came back OK. We don't use it in this example */
    if( (Uint32)hApplication != 0x12345678 ) {
        return;
    }

    /*
     * The status update function is called for a variety of
     * reasons, including LINK change events and EMAC errors.
     * We'll get the current status and print it out. Note we
     * don't start sending packets until we have a good link.
     *
     */
    EMAC_getStatus( hEMAC, &status);

    LinkStatus = status.MdioLinkStatus;

    /* if( !LinkStatus )
        printf("Link Status: %s\n",LinkStr[LinkStatus]);
    else if( LinkStatus <= 4 )
        printf("Link Status  : %s on PHY number %d\n",
               LinkStr[LinkStatus],status.PhyDev);

    printf("Packets Held : %d-RX  %d-TX\n",
           status.RxPktHeld, status.TxPktHeld);
    if( status.FatalError )
        printf("Fatal Error  : %d\n",status.FatalError);
    */
}

/*
 * =============================================================================
 *   @func   StatisticsUpdate
 *
 *   @desc
 *      The EMAC statistics are in danger of overflow
 *
 *   @arg
 *      hApplication    handle to the application
 *      
 *   @eg
 *      Handle   hApp;
 *      ...
 *      StatisticsUpdate(hApp); // Refer description
 *      ...
 * =============================================================================
 */
static void StatisticsUpdate( Handle hApplication )
{
    /* Verify our handle came back OK. We don't use it in this example */
    if( (Uint32)hApplication != 0x12345678 ) {
        return;
    }

    /*
     * Here we could call the EMAC_getStatistics() function and pass
     * them to whoever wants them. This example app doesn't have
     * anything to do with them so we do nothing.
     */
}

/*
 * =============================================================================
 *   @func   verify_packet()
 *
 *   @desc
 *      Perform a sanity check validation on the packet
 *
 *   @arg
 *      pPkt    EMAC packet
 *      size    size to validate
 *      
 *   @eg
 *      EMAC_Pkt    *pkt;
 *      Uint32      size;
 *      ... ; // variables to get value elsewhere.
 *      verify_packet(pkt, size);
 *      ...
 * =============================================================================
 */
static int verify_packet( EMAC_Pkt *pPkt, Uint32 size )
{
    int i;
    Uint8 *pBuf;

    if( (pPkt->Flags & (EMAC_PKT_FLAGS_SOP | EMAC_PKT_FLAGS_EOP)) !=
        (EMAC_PKT_FLAGS_SOP | EMAC_PKT_FLAGS_EOP) ) {
        printf("Verify: Bad Flags %08x\n",pPkt->Flags);
        return(0);
    }

    if( pPkt->ValidLen != size ) {
        printf("Verify: Bad ValidLen %d %d\n",pPkt->ValidLen,size);
        return(0);
    }

    if( pPkt->DataOffset != 0 ) {
        printf("Verify: Bad DataOffset %d\n",pPkt->DataOffset);
        return(0);
    }


    if( pPkt->PktChannel != 0 ) {
        printf("Verify: Bad Channel %d\n",pPkt->PktChannel);
        return(0);
    }

    if( pPkt->PktLength != size ) {
        printf("Verify: Bad PktLength %d\n",pPkt->PktLength);
        return(0);
    }

    if( pPkt->PktFrags != 1 ) {
        printf("Verify: Bad Frag Count %d\n",pPkt->PktFrags);
        return(0);
    }

    pBuf = pPkt->pDataBuffer+pPkt->DataOffset;
	pBuf += 6;
    for( i=6; i<size-6; i++, pBuf++ )
        if( *pBuf != (i&0xFF) ) {
            printf("Verify: Bad data at %d (%d)(%d)\n",i,*pBuf,i&0xff);
            return(0);
        }

    return(1);
}

/* ============================================================================
 *  @func App_pqPop()
 *
 *  @desc
 *      Pop a desc buffer off a queue
 *
 *   @arg
 *      pq    Application packet
 *      
 *   @eg
 *      APP_PKTQ    *pq;
 *      ... ; // variables to get value elsewhere.
 *      App_pqPop(pq);
 *      ...
 * ============================================================================
 */
static EMAC_Pkt *App_pqPop( APP_PKTQ *pq )
{
    EMAC_Pkt *pPktHdr;
   
	OUREMAC_enter();

    pPktHdr = pq->pHead;

    if( pPktHdr ) {
        pq->pHead = pPktHdr->pNext;
        pq->Count--;
        pPktHdr->pPrev = pPktHdr->pNext = 0;
    }

	OUREMAC_exit();

    return( pPktHdr );
}

/* ============================================================================
 *  @func App_pqPush()
 *
 *  @desc
 *      Push a desc buffer onto a queue
 *
 *   @arg
 *      pq      Application packet
 *      pPktHdr pointer to EMAC packet header
 *      
 *   @eg
 *      APP_PKTQ    *pq;
 *      EMAC_Pkt    *pPktHdr;
 *      ... ; // variables to get value elsewhere.
 *      App_pqPush(pq, pPktHdr);
 *      ...
 * ============================================================================
 */
static void App_pqPush( APP_PKTQ *pq, EMAC_Pkt *pPktHdr )
{
	OUREMAC_enter();

    pPktHdr->pNext = 0;

    if( !pq->pHead ) {
        /* Queue is empty - Initialize it with this one packet */
        pq->pHead = pPktHdr;
        pq->pTail = pPktHdr;
    }
    else {
        // Queue is not empty - Push onto END
        pq->pTail->pNext = pPktHdr;
        pq->pTail        = pPktHdr;
    }
    pq->Count++;

	OUREMAC_exit();
}

