/*  ============================================================================
 *   Copyright (c) Texas Instruments Inc 2002, 2003, 2004, 2005, 2008
 *
 *   Use of this software is controlled by the terms and conditions found
 *   in the license agreement under which this software has been supplied.
 *  ============================================================================
 */

/** ============================================================================
 *
 *   @file  Hpi_example.c
 *
 *   @path  $(CSLPATH)\example\hpi\src
 *
 *   @desc  In this example, demonstrates the CSL HPI functionalities. 
 *
 *  ============================================================================
 *   @n Target Platform: 
 *  ============================================================================
 *   @n <b> Example Description </b>
 *   @verbatim
      This Example, 
       1. Enables the powersaver clock of HPI.
       2. Initializes and Opens the HPI CSL module.
       3. Takes HPI out of reset.
       4. Sets Host Interrupt
       5. Queries for peripheral revision Id, class, type.
       6. Queries for host ready.

     @endverbatim
 *
 * =============================================================================
 *      
 *   <b> Procedure to run the example </b>
 *   @verbatim
 *      1. Configure the CCS setup to work with the emulator being used
 *      2. Please refer CCS manual for setup configuration and loading 
 *         proper GEL file
 *      3. Launch CCS window
 *      4. Open project Hpi_example.pjt
 *      5. Build the project and load the .out file of the project.
 *          
 *   @endverbatim
 *   
 * =============================================================================
 */ 
/* =============================================================================
 *  Revision History
 *  ================
 *  10-May-2008 Removed incorrect commands.
 *  26-Mar-2008 Use HPIC register values as 32 bits instead of enum in CSL_HpiHwSetup
 *
 *  21-Sep-2005 NG  Created
 * 
 *  30-Nov-2005 NG  Updated documentation
 *
 *  25-Jan-2006 NG  Updated Example
 *
 *  06-Feb-2006 ds  Changed to use DEV functional layer 
 *  10-Nov-2008 Added PSC functional layer to enable HPI
 * =============================================================================
 */
#include <stdio.h>
#include <csl.h>
#include <csl_hpi.h>
#include <csl_psc.h>


/* HPI Handle Initialize to NULL */
CSL_HpiHandle    hHpi = (CSL_HpiHandle) NULL;
/* CSL status */
CSL_Status    status;
Uint32        response;

/* forward declaration */
void hpi_error_exit(void);
CSL_Status hpi_example(void);
void pwrsvrUhpiClk_enable(void);


/*
 * =============================================================================
 *   @func   main
 *
 *   @desc
 *     This is the main routine for the file.  
 *    
 * =============================================================================
 */
void main (
)
{
   
    pwrsvrUhpiClk_enable();

    status = hpi_example();

    if (status == CSL_SOK) {
        printf ("HPI CSL example complete\n");
    }
    else {
        printf ("HPI CSL example is not complete\n");
    }
}

/*
 * ============================================================================
 *   @func   hpi_example
 *
 *   @desc
 *     This function will initialize the HPI module and also write and read
 *      back the value from the HPI register.
 *
 *  @arg  
 *      NONE
 *
 *  @return
 *      NONE
 * ============================================================================
*/

CSL_Status hpi_example (
    void
)
{
    CSL_HpiObj        hpiObj;
    CSL_HpiHwSetup    hwSetup;
    Uint32            arg;

    /* hardware setup for HPI */
    hwSetup.hpiCtrl = 0x4;
    hwSetup.hpiAddr.hpiaReadAddr = 0x0;
    hwSetup.hpiAddr.hpiaWrtAddr = 0x0;

    /* Initialiaze device config registers related to HPI */
    status = CSL_hpiInit(NULL);
    if (status != CSL_SOK) 
	{
        printf("\nNot Initialized");
        hpi_error_exit();
        return status;
    }

    /* Open the hpi CSL module */
    hHpi = CSL_hpiOpen(&hpiObj, CSL_HPI, NULL, &status);
    if ((hHpi == NULL) || (status != CSL_SOK)) 
	{
        printf("HPI: Error opening the instance. [status = 0x%x, hHpi \
                = 0x%x]\n", status, hHpi);      
        hpi_error_exit();
        return status;
    }

    status = CSL_hpiHwSetup(hHpi, &hwSetup);
    if (status != CSL_SOK) 
	{
        printf("HPI: Hardware setup is NOT successfull\n");
        hpi_error_exit();
        return status;        
    }
    else 
        printf("HPI: Hardware setup is successfull\n");
    
    /* Set Host Interrupt */
    arg = 1;
    status  = CSL_hpiHwControl(hHpi, CSL_HPI_CMD_SET_HINT, &arg);
    if (status != CSL_SOK) 
	{
        printf("HPI: Not able to set Host Interrupt.\n");
        hpi_error_exit();
        return status;        
    }
    else
        printf("HPI: Host Interrupt set \n");
   
	
    /* Reset HPI */
    arg = 1;
    status  = CSL_hpiHwControl(hHpi, CSL_HPI_CMD_RESET_HPI, &arg);
    if (status != CSL_SOK) 
	{
        printf("HPI: Not able to reset HPI.\n");
        hpi_error_exit();
        return status;        
    }
    else 
        printf("HPI: HPI reset successfully.\n");
    


    hpi_error_exit();
    return status;
}

/******************************************************************************
* @func hpi_error_exit
*
* @desc Error in executing the example. As error is occurred closing the HPI
*       handle and returning.
*
* @arg  
*   NONE
*
* @return
*   NONE
******************************************************************************/
void hpi_error_exit (
    void
)
{
    status = CSL_hpiClose(hHpi);
    if (status != CSL_SOK)
        printf("HPI:Unable to close the instance\n");
    return;
}

/*
 * =============================================================================
 *   @func   pwrsvrUhpiClk_enable
 *
 *   @arg
 *      NONE
 *
 *   @desc
 *     This function enables the powersaver clock of HPI.
 *
 *   @return
 *      NONE
 *
 * =============================================================================
 */
void pwrsvrUhpiClk_enable (
)
{
    CSL_PscObj pscObj;
    CSL_PscHandle hPsc;
    CSL_Status status;
    CSL_PscPowerDomain pwrDmn = CSL_PSC_PWRDMN_ALWAYSON;
    CSL_PscPeripherals module = CSL_PSC_MODULE_HPI;
    CSL_PscPwrDmnTransState response;

    memset(&pscObj, 0, sizeof(CSL_PscObj));      

    /* Init PSC module */
    CSL_pscInit(NULL);

    /* Open PSC module to get a Handle */
    hPsc = CSL_pscOpen(&pscObj, CSL_PSC, NULL, &status);

    /* Enable power domain for HPI */
    CSL_pscHwControl(hPsc, CSL_PSC_CMD_ENABLE_PWRDMN, &pwrDmn);

    /* Enable clock for the specified module HPI */
    CSL_pscHwControl(hPsc, CSL_PSC_CMD_ENABLE_MODULE, &module);
	
    /* Enable HPI Power domain GO transition */
    CSL_pscHwControl(hPsc, CSL_PSC_CMD_PWRDMN_TRNS, &pwrDmn);

    response.pwrDmn = CSL_PSC_PWRDMN_ALWAYSON;
    response.status = 0x0; 
    do{
        CSL_pscGetHwStatus(hPsc, CSL_PSC_QUERY_PWRDMN_TRANS_STAT, (void *)&response);
        (response.status) &= (1 << CSL_PSC_PWRDMN_ALWAYSON);
    }while((response.status) != 0x0);

    /* Close the PSC module */
    CSL_pscClose(hPsc);
}

