/*  ============================================================================
 *   Copyright (c) Texas Instruments Inc 2002, 2003, 2004, 2005, 2006
 *
 *   Use of this software is controlled by the terms and conditions found
 *   in the license agreement under which this software has been supplied.
 *  ============================================================================
 */
/** ============================================================================
 *   @file  i2c_edma_example.c
 *
 *   @path  $(CSLPATH)\example\c6488\i2c\i2c_edma_example\src
 *
 *   @desc  Example of i2c
 *
 *  ============================================================================
 *   @n Target Platform: VDB
 *  ============================================================================
 *   @n <b> Example Description </b>
 *   @n The example shows the functionality of I2C in Loop back mode
 *          - Enables the I2C power saver clock
 *          - Initalizes and Opens I2C instance. 
 *          - Takes i2c out of reset
 *          - Configures the EDMA for TX & RX
 *          - Enables NMIs and Global Interrupts
 *          - Enables and starts I2C
 *          - Waits for the transfer to start
 *          - verifies the data received 
 *          - Stops the transmission and
 *          - Closes the opened instance
 *
 *==============================================================================
 *      
 *   <b> Procedure to run the example </b>
 *   @verbatim
 *      1. Configure the CCS setup to work with the emulator being used
 *      2. Please refer CCS manual for setup configuration and loading 
 *         proper GEL file
 *      3. Launch CCS window
 *      4. Open project I2c_edma_example.pjt
 *      5. Build the project and load the .out file of the project.
 *          
 *   @endverbatim
 *   
 * =============================================================================
 */ 
/** =============================================================================
 *  Revision History
 *  ===============*
 *  25-Sep-2006 NG  File Created
 * 28-July-2008 Bug fix for i2c timing issues
 * =============================================================================
 */
#include <csl_edma3.h>
#include <csl_i2c.h>
#include <soc.h>
#include <stdio.h>
#include <csl_i2cAux.h>


/* Define data count */
#define DATATX_COUNT        	   	16
#define TEST_ACNT 					1
#define TEST_BCNT 					16
#define TEST_CCNT 					1
   
#define CSL_I2C_MASTER_ADDR        	0x2AA    
#define CSL_I2C_SLAVE_ADDR         	0x2AA   

#define CSL_I2C_CLOCK_PRESCALAR    	0x13  
#define CSL_I2C_CLOCK_CLKL         	0x4   
#define CSL_I2C_CLOCK_CLKH         	0x4

#define dstBufI2CTx					0x02B04020
#define srcBufI2CRx 				0x02B04018

#define CSL_EDMA3_CHA_45 			45
#define CSL_EDMA3_CHA_44 			44
#define NO_INTERRUPTS_ALL_MSK       0x00000000

CSL_Edma3Handle 	 	hModule;
CSL_Edma3ParamHandle 	paramHandle0;
CSL_Edma3ParamHandle 	paramHandle1;
CSL_Edma3ChannelAttr  	chParam;
CSL_Edma3ChannelObj  	ChObj0, ChObj1;
CSL_Edma3ChannelHandle  hChannel0, hChannel1;
CSL_Edma3HwDmaChannelSetup chSetup;
CSL_Edma3ParamSetup 	paramSetup; 
CSL_Edma3Obj  			moduleObj;
CSL_Edma3CmdIntr 		regionIntr;
CSL_Edma3ChannelErr 	chErrClear;
CSL_Status 				EdmaStat;


/* Handle for the I2C instance */
CSL_I2cHandle   hI2c;
CSL_I2cObj      i2cObj;

Uint8          	srcBuff[DATATX_COUNT];
Uint8          	dstBuff[DATATX_COUNT];

/* clk set up */
CSL_I2cClkSetup clksetup = {
    CSL_I2C_CLOCK_PRESCALAR,
    CSL_I2C_CLOCK_CLKL,
    CSL_I2C_CLOCK_CLKH,
};


Uint32   	CountStopper = 0xffff;
Uint32		BbResponse;
CSL_Status     handleStatus;
CSL_I2cConfig  	i2c_config, default_config;	


void i2c_edma_example(void);
void i2c_edma_setup (void);
void i2c_setup(void);
void EDMA3_MPPA_ACCESS ();
void i2c_reset(void);
void delay_counts(Uint32 delay_cnt);

/*
 * =============================================================================
 *   @func   main
 *
 *   @desc
 *     This is the main routine, which invokes I2C edma example scripts
 * =============================================================================
 */
void main (
    void    
)
{   
    i2c_edma_example();
    return;
}

void i2c_edma_example()
{
    Uint32 index;

    /* initializes the transmit buffer */
    for (index = 0; index < DATATX_COUNT; index++) 
    {
        srcBuff[index] = 0x41 + index;
        dstBuff[index] = 0;  
    }

	i2c_setup();
    EDMA3_MPPA_ACCESS();

	i2c_edma_setup();

	/* Stop the transmission */
	if (CSL_i2cHwControl(hI2c, CSL_I2C_CMD_STOP, NULL) != CSL_SOK)
		printf("Unable to stop i2c transmission.\n");

	if (CSL_i2cClose(hI2c) != CSL_SOK)
		printf("Unable to close i2c.\n");	

	/* Check if data is transferred correctly */
    for (index = 0; index < DATATX_COUNT; index++) 
    {
        if (srcBuff[index] != dstBuff[index]) 
        {
            printf("Not mached: RxData = %d\t txData=%d\n Test FAILED...\n", \
                   dstBuff[index],srcBuff[index]);
			return;
		}
		else
		{
			printf("Matched   : RxData = %d\t txData=%d\n Test PASSED...\n", \
                   dstBuff[index],srcBuff[index]);
		}
	}
}

void i2c_setup(void)
{
	CSL_Status      status = CSL_SOK;

	memset(&i2cObj, 0, sizeof(CSL_I2cObj));

	    /* Initialize I2C module */
    status = CSL_i2cInit(NULL);
    if (status != CSL_SOK)
    {
        printf("I2C: Initialization error.\n");
        printf("\tReason: CSL_i2cInit [status = 0x%x].\n", status);
        return;
    }
    
    /* open i2c */
    hI2c = CSL_i2cOpen(&i2cObj, CSL_I2C, NULL, &status);
    if ((status != CSL_SOK) || (hI2c == NULL))
    {
        printf("I2C: Error opening the instance. \
               [status = 0x%x, hI2c = 0x%x]\n", status, hI2c);
        return;
    }

	default_config.ICOAR = CSL_I2C_ICOAR_RESETVAL;	/** 0x0000	Own address register      */
	default_config.ICIMR = CSL_I2C_ICIMR_RESETVAL;	/** 0x0000	Interrupt Mask register   */
	default_config.ICSTR = CSL_I2C_ICSTR_RESETVAL;	/** 0x0410	Status register           */
	default_config.ICCLKL = CSL_I2C_ICCLKL_RESETVAL;	/** 0x0000	Clock low register        */
	default_config.ICCLKH = CSL_I2C_ICCLKH_RESETVAL;	/** 0x0000	clock high register       */
	default_config.ICCNT = CSL_I2C_ICCNT_RESETVAL;	/** 0x0000	Data Count register       */
	default_config.ICSAR = CSL_I2C_ICSAR_RESETVAL;	/** 0x03ff	Slave address register    */
	default_config.ICDXR = CSL_I2C_ICDXR_RESETVAL;	/** 0x0000	Data Transmit register    */
	default_config.ICMDR = CSL_I2C_ICMDR_RESETVAL;	/** 0x0000	Data Receive register     */
	default_config.ICIVR = CSL_I2C_ICIVR_RESETVAL;	/** 0x0000	Interrupt vector register */
	default_config.ICEMDR = CSL_I2C_ICEMDR_RESETVAL;	/** 0x0001	Extended mode register    */
	default_config.ICPSC = CSL_I2C_CLOCK_PRESCALAR;	/** 0x0000	Prescalar register        */

	status = CSL_i2cHwSetupRaw(hI2c, &default_config);
    if ((status != CSL_SOK) || (hI2c == NULL))
	{
        printf("I2C: Error setting Default Config \
               [status = 0x%x, hI2c = 0x%x]\n", status, hI2c);
        return;
    }

	/* let I2c Bus settle if being reset */
	delay_counts(50);

	/* Note: since loopback, setting ICOAR the same as ICSAR */
	i2c_config.ICOAR = CSL_I2C_MASTER_ADDR;		/** I2C Own address register */
	i2c_config.ICIMR = NO_INTERRUPTS_ALL_MSK;	/** I2C Interrupt Mask register */
	i2c_config.ICCLKL = CSL_I2C_CLOCK_CLKL; 	/** I2C Clock low register */
	i2c_config.ICCLKH = CSL_I2C_CLOCK_CLKH; 	/** I2C clock high register */
	i2c_config.ICCNT = CSL_I2C_ICCNT_RESETVAL;	/** I2C Data Count register */
	i2c_config.ICSAR = CSL_I2C_MASTER_ADDR;		/** I2C Slave address register     */
	i2c_config.ICPSC = CSL_I2C_CLOCK_PRESCALAR; /** I2C Prescalar register */
	i2c_config.ICMDR = CSL_I2C_ICMDR_RESETVAL; 

	status = CSL_i2cHwSetupRaw(hI2c, &i2c_config);
    if ((status != CSL_SOK) || (hI2c == NULL))
	{
        printf("I2C: Error setting i2c Config \
               [status = 0x%x, hI2c = 0x%x]\n", status, hI2c);
        return;
    }


	/* let I2c Bus settle if being reset */
	delay_counts(50);

	status = CSL_i2cHwControl(hI2c, CSL_I2C_CMD_SET_DATA_COUNT, &CountStopper);
    if ((status != CSL_SOK) || (hI2c == NULL))
	{
        printf("I2C: Error setting datacount \
               [status = 0x%x, hI2c = 0x%x]\n", status, hI2c);
        return;
    }

	do 
	{
	    CSL_i2cGetHwStatus(hI2c, CSL_I2C_QUERY_BUS_BUSY, &BbResponse);
	} while(BbResponse != 0); /* will not hit here if return on failure */
}

void EDMA3_MPPA_ACCESS () 
{
  	Uint16 * mem_ptr;
    for (mem_ptr = (Uint16*)(0x02A0080C);  mem_ptr < (Uint16*)(0x02A0082C); mem_ptr++) 
      *(mem_ptr) = 0xFFF0;

}

void i2c_edma_setup()
{

    // EDMA Module Initialization
	CSL_edma3Init(NULL);
    
	// EDMA Module Open	
	hModule = CSL_edma3Open(&moduleObj,CSL_EDMA3,NULL,&EdmaStat);

	// Tx Channel Open - Channel 45 for Tx
	chParam.regionNum  = CSL_EDMA3_REGION_GLOBAL;
	chSetup.que        = CSL_EDMA3_QUE_2;
	chParam.chaNum     = CSL_EDMA3_CHA_45;
	hChannel0          = CSL_edma3ChannelOpen(&ChObj0,
	                       CSL_EDMA3,
	                       &chParam,                            
	                       &EdmaStat);	
	                    
	// Channel Setup 
	chSetup.paramNum   = CSL_EDMA3_CHA_45;	
   	CSL_edma3HwChannelSetupQue(hChannel0, chSetup.que);
    CSL_edma3HwChannelSetupParam(hChannel0, chSetup.paramNum);

	// Rx Channel Open - Channel 44 for Rx
	chParam.regionNum  = CSL_EDMA3_REGION_GLOBAL;
	chSetup.que        = CSL_EDMA3_QUE_2;
	chParam.chaNum     = CSL_EDMA3_CHA_44;
	hChannel1          = CSL_edma3ChannelOpen(&ChObj1,
	                       CSL_EDMA3,
	                       &chParam,                            
	                       &EdmaStat);	
	                    
	// Channel Setup 
	chSetup.paramNum = CSL_EDMA3_CHA_44;	
   	CSL_edma3HwChannelSetupQue(hChannel1, chSetup.que);
    CSL_edma3HwChannelSetupParam(hChannel1, chSetup.paramNum);

	// Parameter Handle Open
	// Open all the handles and keep them ready
  
  	paramHandle0            = CSL_edma3GetParamHandle(hChannel0, CSL_EDMA3_CHA_45, &handleStatus);
	paramHandle1            = CSL_edma3GetParamHandle(hChannel1, CSL_EDMA3_CHA_44, &handleStatus);
  
    paramSetup.aCntbCnt     = CSL_EDMA3_CNT_MAKE(TEST_ACNT, TEST_BCNT);       		
	paramSetup.srcDstBidx   = CSL_EDMA3_BIDX_MAKE(TEST_ACNT, 0 );  
	paramSetup.srcDstCidx   = CSL_EDMA3_CIDX_MAKE(0, 0);     		
	paramSetup.cCnt         = TEST_CCNT;
	paramSetup.option       = CSL_EDMA3_OPT_MAKE(FALSE, FALSE, FALSE, TRUE, CSL_EDMA3_CHA_45, CSL_EDMA3_TCC_NORMAL, \
	      CSL_EDMA3_FIFOWIDTH_NONE, FALSE, CSL_EDMA3_SYNC_A, CSL_EDMA3_ADDRMODE_INCR, CSL_EDMA3_ADDRMODE_INCR);           
	paramSetup.srcAddr      = (Uint32)srcBuff;         
	paramSetup.dstAddr      = (Uint32)dstBufI2CTx;
	paramSetup.linkBcntrld  = CSL_EDMA3_LINKBCNTRLD_MAKE(CSL_EDMA3_LINK_NULL, 0);     

	CSL_edma3ParamSetup(paramHandle0, &paramSetup);
	
    paramSetup.aCntbCnt     = CSL_EDMA3_CNT_MAKE(TEST_ACNT, TEST_BCNT);       		
	paramSetup.srcDstBidx   = CSL_EDMA3_BIDX_MAKE(0, TEST_ACNT );  
	paramSetup.srcDstCidx   = CSL_EDMA3_CIDX_MAKE(0, 0);     		
	paramSetup.cCnt         = TEST_CCNT;
	paramSetup.option       = CSL_EDMA3_OPT_MAKE(FALSE, TRUE, FALSE, TRUE, CSL_EDMA3_CHA_44, CSL_EDMA3_TCC_NORMAL, \
	      CSL_EDMA3_FIFOWIDTH_NONE, FALSE, CSL_EDMA3_SYNC_A, CSL_EDMA3_ADDRMODE_INCR, CSL_EDMA3_ADDRMODE_INCR);           
	paramSetup.srcAddr      = (Uint32)srcBufI2CRx;         
	paramSetup.dstAddr      = (Uint32)dstBuff;
	paramSetup.linkBcntrld  = CSL_EDMA3_LINKBCNTRLD_MAKE(CSL_EDMA3_LINK_NULL, 0);     
    
	CSL_edma3ParamSetup(paramHandle1,&paramSetup);

   	/* Trigger channel */
   	CSL_edma3HwChannelControl(hChannel0, CSL_EDMA3_CMD_CHANNEL_ENABLE, NULL);
   	CSL_edma3HwChannelControl(hChannel1, CSL_EDMA3_CMD_CHANNEL_ENABLE, NULL);

	/* Bring I2C out of reset. This should be the last step */
	i2c_reset();

	/* Wait for interrupt */
    regionIntr.region  = CSL_EDMA3_REGION_GLOBAL;
	regionIntr.intr    = 0;
	regionIntr.intrh   = 0;
	do{
		CSL_edma3GetHwStatus(hModule, CSL_EDMA3_QUERY_INTRPEND, &regionIntr);
	}while ((regionIntr.intrh & 0x3000) != 0x3000);// channel 44 & 45

	CSL_FINST(hModule->regs->SECR,EDMA3CC_SECRH_E44, CLEAR);
  	CSL_FINST(hModule->regs->SECR,EDMA3CC_SECRH_E45, CLEAR);

	/* close instance of EDMA */   
    CSL_edma3ChannelClose(hChannel0);
    CSL_edma3ChannelClose(hChannel1);
    CSL_edma3Close(hModule);
}

void i2c_reset(void)
{
	i2c_config.ICMDR = CSL_I2C_ICMDR_NACKMOD_RESETVAL | CSL_I2C_ICMDR_FREE_RESETVAL | CSL_I2C_ICMDR_STT_MASK | 
		CSL_I2C_ICMDR_STP_MASK | CSL_I2C_ICMDR_MST_MASK | CSL_I2C_ICMDR_TRX_MASK | CSL_I2C_ICMDR_XA_RESETVAL | CSL_I2C_ICMDR_RM_RESETVAL | CSL_I2C_ICMDR_DLB_MASK |
		CSL_I2C_ICMDR_IRS_MASK | CSL_I2C_ICMDR_STB_RESETVAL | CSL_I2C_ICMDR_FDF_RESETVAL | 	 CSL_I2C_ICMDR_BC_RESETVAL;
				
	if (CSL_i2cHwSetupRaw(hI2c, &i2c_config) != CSL_SOK)
	{
        printf("I2C: Error bringing i2c out of reset \
               [hI2c = 0x%x]\n", hI2c);
        return;
    }
}

void delay_counts(Uint32 delay_cnt)
{
	Uint32	delay_ctr, i;
	delay_ctr = 0;
	/* avoid a hanging if count is zero */
	if(delay_cnt == 0)
	return; 
	/* let I2c Bus settle if being reset */
	for (i = 0; i <= delay_cnt ; i++)
	{
		delay_ctr++;
	} /* end... for  */
} /* end... delay_counts(Uint32 delay_cnt) */


