/*  ============================================================================
 *   Copyright (c) Texas Instruments Inc 2002, 2003, 2004, 2005, 2008
 *
 *   Use of this software is controlled by the terms and conditions found in the
 *   license agreement under which this software has been supplied.
 *   ===========================================================================
 */

/* ============================================================================
 *   @file  pwrdwnExample.c
 *
 *   @path $(CSLPATH)\example\pwrdwn\src
 *
 *   @desc  Example for pwrdwn module
 *
 *  ============================================================================
 *   @n Target Platform: 
 *  ============================================================================
 *   @n <b> Example Description </b>
 *      This function invokes example that prove functionalites of power down 
 *      controller
 *   @n This example, 
 *        1. Initializes and opens the CSL PWRDWN module instance.
 *        2. Sets up the hardware to setup parameters i.e.,  
 *           CSL_pwrdwnHwSetup() is called for module configuration.
 *        3. Set the megamodule mode to sleep mode
 *        4. Gets the megamodule status
 *        5. Set the megamodule mode to normal mode
 *        6. Gets the megamodule status
 *
 * =============================================================================
 *      
 *   <b> Procedure to run the example </b>
 *   @verbatim
 *      1. Configure the CCS setup to work with the emulator being used
 *      2. Please refer CCS manual for setup configuration and loading 
 *         proper GEL file
 *      3. Launch CCS window
 *      4. Open project Pwrdwn_example.pjt
 *      5. Build the project and load the .out file of the project.
 *          
 *   @endverbatim
 *
 */

/* =============================================================================
 * Revision History
 * ================
 *  23-Apr-2008 Removed L2 PWRDWN and added PDCCMD MEGPD.
 *  28-Jul-2005 PSK  File Created
 *  16-Dec-2005 ds   Updated documentation
 * =============================================================================
 */

#include <csl_pwrdwn.h>
#include <soc.h>
#include <stdio.h>

void pwrdwn_example(void);

Uint32 demoFail = 0;

/*
 * =============================================================================
 *  @func   main
 *
 *  @desc
 *     This is the main routine for the file.
 *
 *  @expected result
 *      If the example passes, it displays the message "PASSED"
 *      If the example fails, it displays the messae "FAILED" 
 *
 * =============================================================================
*/
void main(void)
{
    
    printf("PWRDWN EXAMPLE DEMO\n");
    
    /* Invoke example */
    pwrdwn_example();
    
    if (demoFail > 0) {
        printf("\nPWRDWN:  example Demo... FAILED.\n");
    }
    else {
        printf("\nPWRDWN: example Demo... PASSED\n");
    }

    return;
}

/*
 * =============================================================================
 *   @func   pwrdwn_example
 *
 *   @desc
 *      This function invokes example that prove functionalites of power down 
 *      controller. This example puts a page on a specific port into sleep 
 *      and verifies the status for that page by calling the CSL API to 
 *      retrieve the status parameters.
 *            
 *   @arg
 *      None
 *  
 *   @eg
 *      pwrdwn_example();
 * =============================================================================
*/
void pwrdwn_example (void)
{
    CSL_PwrdwnContext   pContext;
    CSL_PwrdwnObj       pwrdwnObj;
    CSL_PwrdwnHandle    hPwrdwn;
    CSL_Status          status;
    CSL_PwrdwnHwSetup   hwSetup;
    Uint8				response;
    
   
    hwSetup.idlePwrdwn = (Bool)0x0;
    
    /* Initialize the module */
    if (CSL_pwrdwnInit(&pContext) != CSL_SOK)
	{
        printf("PWRDWN: Initialization...  FAILED\n");
        demoFail++;
    }
    
    /* Clear the local data structures */
    memset(&pwrdwnObj, 0, sizeof(CSL_PwrdwnObj));

    /* Open the module */
    hPwrdwn = CSL_pwrdwnOpen (&pwrdwnObj, (CSL_InstNum)CSL_PWRDWN, NULL, 
                             &status);
    if ((hPwrdwn == NULL) || (status != CSL_SOK)) 
	{
        printf("PWRDWN: Error in opening the instance ...\n");
        demoFail++;
    }
    
    /* Do hardware setup */
    if (CSL_pwrdwnHwSetup(hPwrdwn, &hwSetup) != CSL_SOK) 
	{
        printf("PWRDWN: CSL_pwrdwnHwSetup ... FAILED\n");
        demoFail++;
    }
    
                                                                
    /* Set the megamodule mode to sleep mode */
    if (CSL_pwrdwnHwControl(hPwrdwn,CSL_PWRDWN_CMD_PDCCMD_MEGPD_SLEEP, NULL) \
                                                                 != CSL_SOK) 
	{
        printf("PWRDWN: CSL_pwrdwnHwControl              ... FAILED\n");
        demoFail++; 
    }
        
            
    /* Get the status of megamodule mode */    
    if (CSL_pwrdwnGetHwStatus (hPwrdwn, CSL_PWRDWN_QUERY_MEGPD_STATUS, 
                                            &response) != CSL_SOK)
	{
        printf ("PWRDWN: CSL_pwrdwnGetHwStatus              ... FAILED\n");
        demoFail++; 
    }
    
    /* Check for the correct status of the page */
    if ((response & 0x1) == 0x1)
        printf("Status example PASSED\n");
        
     /* Set the megamodule mode to normal mode */
    if (CSL_pwrdwnHwControl(hPwrdwn,CSL_PWRDWN_CMD_PDCCMD_MEGPD_NORMAL, NULL) \
                                                                 != CSL_SOK) 
	{
        printf("PWRDWN: CSL_pwrdwnHwControl              ... FAILED\n");
        demoFail++; 
    }
        
            
    /* Get the status of megamodule mode */    
    if (CSL_pwrdwnGetHwStatus (hPwrdwn, CSL_PWRDWN_QUERY_MEGPD_STATUS, 
                                            &response) != CSL_SOK) 
	{
        printf ("PWRDWN: CSL_pwrdwnGetHwStatus              ... FAILED\n");
        demoFail++; 
    }
    

    if (response == 0) 
        printf("PWRDWN: functionality example PASSED");    
    else
	{
        printf("PWRDWN: functionality example FAILED");
        demoFail++;
	}
    return;
}
