/* ============================================================================
 * Copyright (c) Texas Instruments Inc 2002, 2003, 2004, 2005, 2008
 *
 * Use of this software is controlled by the terms and conditions found in the
 * license agreement under which this software has been supplied.
 * ============================================================================
 */

/** ===========================================================================
 *   @file Srio_ext_loopback_example.c
 *
 *   @path  $(CSLPATH)\example\srio\src
 *
 *   @desc This is a simple external lookback example of usage of CSL APIs 
 *         of the SRIO module.
 *
 *  ============================================================================
 *   @n Target Platform: 
 *  ============================================================================
 *
 * ============================================================================
 *   @n <b> Example Description </b>
 *   @n This is an example of the CSL SRIO usage for a LSU port.
 *   @verbatim
       1. Setup SRIO module for external loop back transfer on LSU1.
       2. Setup the destination and source data areas.
       3. Configure LSU1 registers.
       4. Start transfer.
       5. Wait for completion of the transfer.
       6. Verify the destination contains the expected data.

     @endverbatim
 *
 *=============================================================================
 *      
 *   @n <b> Procedure </b>
 *   @verbatim
      1. Configure the CCS setup
      2. Please refer CCS manual for setup configuration and loading 
         proper GEL file
      3. Launch CCS window
      4. Open project Srio_ext_loopback_example.pjt
      5. Build the project and load the .out file of the project.

     @endverbatim
 *       
 *   
 * ============================================================================
 */

/* ============================================================================
 * Revision History
 * ===============
 * 31-Aug-2006 NG  File created
 * 07-July-2008 Update to include Faraday changes 
 *  10-Nov-2008 Added PSC functional layer to enable SRIO
 * ============================================================================
 */
#include <csl_srio.h>
#include <soc.h>
#include <csl_psc.h>
#include <stdio.h>
#include <string.h>

/* Macros */

#define SRIO_SET_DEVICE_ID(base_dev_id, large_base_dev_id)              \
            CSL_FMK(SRIO_BASE_ID_BASE_DEVICEID, base_dev_id) |          \
            CSL_FMK(SRIO_BASE_ID_LARGE_BASE_DEVICEID, large_base_dev_id)

#define LARGE_DEV_ID            0xBEEF
#define SMALL_DEV_ID            0xAB

/* Definition for the NWRITE_R RIO Command. */
#define SRIO_PKT_TYPE_NWRITE_R	0x55

#define SELECTED_LSU            0

#define TRANSFER_SIZE           256

/* SRIO Port Number on which the test will execute. */
#define SRIO_PORT_NUMBER        0

#define REQ_MAINT_WR		    0x81 //0b10000001  // ftype=8

/* globals */
static Uint8 src[TRANSFER_SIZE];
static Uint8 dst[TRANSFER_SIZE];
static CSL_SrioHwSetup setup = CSL_SRIO_HWSETUP_DEFAULTS;

/* Function prototypes */
void srio_Create_Setup (
    CSL_SrioHwSetup *pSetup,
	int bootcomplete
);

void enableSRIO(void);

/*
 * ============================================================================
 *   @func   srio_start_maintenance
 *
 *   @desc
 * ============================================================================
 */
srio_start_maintenance (CSL_SrioHandle hSrio, int xmtAddress, int rcvAddress, int byte_count, int type, int port, int Lsuno)
{
		
    hSrio->regs->LSU[Lsuno].LSU_REG0 = 	CSL_FMK( SRIO_LSU_REG0_ADDRESS_MSB,0 );			
    hSrio->regs->LSU[Lsuno].LSU_REG1 = 	CSL_FMK( SRIO_LSU_REG1_ADDRESS_LSB_CONFIG_OFFSET, rcvAddress ); //32bit = type 2,5,6. 24bit = type 8
    hSrio->regs->LSU[Lsuno].LSU_REG2 = 	CSL_FMK( SRIO_LSU_REG2_DSP_ADDRESS, xmtAddress); //&xmtbuffer
    hSrio->regs->LSU[Lsuno].LSU_REG3 = 	CSL_FMK( SRIO_LSU_REG3_BYTE_COUNT,byte_count );							
    hSrio->regs->LSU[Lsuno].LSU_REG4 = 	CSL_FMK( SRIO_LSU_REG4_OUTPORTID,port )|				
                                        CSL_FMK( SRIO_LSU_REG4_PRIORITY,0 )|				
                                        CSL_FMK( SRIO_LSU_REG4_XAMBS,0 )|					
                                        CSL_FMK( SRIO_LSU_REG4_ID_SIZE,1 )|				
                                        CSL_FMK( SRIO_LSU_REG4_DESTID,0xBEEF )|		
                                        CSL_FMK( SRIO_LSU_REG4_INTERRUPT_REQ,1 );		
    hSrio->regs->LSU[Lsuno].LSU_REG5 = 	CSL_FMK( SRIO_LSU_REG5_DRBLL_INFO,0x0000 )|		
                                        CSL_FMK( SRIO_LSU_REG5_HOP_COUNT,0x00 )|		
                                        CSL_FMK( SRIO_LSU_REG5_PACKET_TYPE,type );		
}

/*
 * ============================================================================
 *   @func   ackID_sync
 *
 *   @desc
 * ============================================================================
 */
void ackID_sync (CSL_SrioHandle hSrio)
{
    Uint32              count, response, ackID, response_valid;
    volatile Int32 		LSUNO = 1;
    Uint32 				rcvAddress;
    Uint32 				partner_inbound_ackid;
    
    response_valid = 0;

    while (!response_valid)
    {
        /***** Send Link Request Control Symbols *****/
        hSrio->regs->PORT[SRIO_PORT_NUMBER].SP_LM_REQ = 0x04;
    
        for (count=0; count<1000; count++)
            asm(" nop 5");

        response = hSrio->regs->PORT[SRIO_PORT_NUMBER].SP_LM_RESP;
        printf ("Response = 0x%08x\n", response);

        response_valid = (response & 0x80000000) >> 31;
    }
    ackID = (response & 0x3E0) >> 5;

    /***** Copy the expected INBOUND_ACKID sent by other device into OUTBOUND_ACKID *****/
    hSrio->regs->PORT[SRIO_PORT_NUMBER].SP_ACKID_STAT = (hSrio->regs->PORT[SRIO_PORT_NUMBER].SP_ACKID_STAT & 0xFFFFFFE0) | ackID;

    /***** Send Maintenance packet to link partner to set his OUTBOUND/INBOUND ACKID  *****/
    /***Value written should be: outstanding = outbound = DSPs expected inbound ACKID
        value, and the inbound = 1 + ACKID from the LM response above***/

    /* little endian, so not shifted */
    partner_inbound_ackid = ackID + 1;

    rcvAddress = 0x00000168;
    srio_start_maintenance(hSrio, (int) &partner_inbound_ackid,(int) rcvAddress, 4, REQ_MAINT_WR, SRIO_PORT_NUMBER,LSUNO);
    printf ("Partner_Inbound_AckID = 0x%08x\n", partner_inbound_ackid);
}

/*
 * ============================================================================
 *   @func   main
 *
 *   @desc
 *     This is the main routine of this program which invokes the different
 *     components to complete a simple LSU transfer.
 * ============================================================================
*/
void main (
    void
)
{

    volatile Uint32 index, dummy_cnt;
    Uint8 lsu_no;

    CSL_SrioContext context;
    CSL_Status status;

    CSL_SrioHandle hSrio;
    CSL_SrioObj   srioObj;
    CSL_InstNum   srioNum = 0;  /* Instance number of the SRIO */
    CSL_SrioParam srioParam;

    CSL_SrioDirectIO_ConfigXfr lsu_conf = {0};
    CSL_SrioPortData response;

    /* Enable SRIO in PSC control module */
	enableSRIO();
 
    /* Initialization and Open of the SRIO */
    status = CSL_srioInit (&context);
    hSrio = CSL_srioOpen (&srioObj, srioNum, &srioParam, &status);
    if (status != CSL_SOK) {
        printf("SRIO: ... Cannot open SRIO, failed\n");
        return;
    }

    /* Create the setup structure */
    srio_Create_Setup (&setup, 1);

    response.index = 0;
    do {
	    /* Setup the SRIO with the selected setup in the last step */
    	status = CSL_srioHwSetup (hSrio, &setup);
    	if (status != CSL_SOK) {
       		printf("SRIO: ... Hardwrae setup, failed\n");
        	return;
    	}

		//Delay for PLL? need to re-clear errors?
		for(index=0; index<0x10000; index++) {
       		dummy_cnt = index + 1;
			dummy_cnt++;
		}

        CSL_srioGetHwStatus (hSrio, CSL_SRIO_QUERY_SP_ERR_STAT, &response);
    }while(response.data & 0x1);

    ackID_sync (hSrio);

    /* Setup the source and destination */
    for(index=0; index<TRANSFER_SIZE; index++) {
	    src[index] = index + 1;
	    dst[index] = 0;
	}

    /* Create an LSU configuration */
    lsu_conf.srcNodeAddr           = (Uint32)&src[0]; /* Source address */
    lsu_conf.dstNodeAddr.addressHi = 0;
    lsu_conf.dstNodeAddr.addressLo = (Uint32)&dst[0]; /* Destination address */
    lsu_conf.byteCnt               = TRANSFER_SIZE;
    lsu_conf.idSize                = 1;               /* 16 bit device id */
    lsu_conf.priority              = 2;               /* PKT priority is 2 */
    lsu_conf.xambs                 = 0;               /* Not an extended
                                                          address */
    lsu_conf.dstId                 = LARGE_DEV_ID;
    lsu_conf.intrReq               = 0;               /* No interrupts */
    lsu_conf.pktType               = SRIO_PKT_TYPE_NWRITE_R;
                                                      /* write with no
                                                          response */
    lsu_conf.hopCount              = 0;               /* Valid for
                                                          maintainance pkt */
    lsu_conf.doorbellInfo          = 0;               /* Not a doorbell pkt */
    lsu_conf.outPortId             = 0;               /* Tx on Port 0 */

	for(index=0; index<0x10000; index++) {
    /* Configure the LSU1 and start transmission */
    lsu_no = SELECTED_LSU;
    CSL_srioLsuSetup (hSrio, &lsu_conf, lsu_no);

    /* Wait for the completion of transfer */
    response.index = lsu_no;
    do {
        CSL_srioGetHwStatus (hSrio, CSL_SRIO_QUERY_LSU_BSY_STAT, &response);
    }while(response.data == 1);
	}

	/* A delay, above checks seems to be not working
	   may be because, it is a write operation with
	   no response */
	for(index=0; index<0x10000; index++) {
        dummy_cnt = index + 1;
		dummy_cnt++;
	}

    /* Verify whether data is transfered */
    for(index=0; index<TRANSFER_SIZE; index++) {
	    if (src[index] != dst[index]) {
		    break;
        }
    }
    if (index == TRANSFER_SIZE) {
	    printf("SRIO: ... LSU transfered %d bytes of data correctly\n",
	        TRANSFER_SIZE);
    } else {
	    printf("SRIO: ... LSU failed to transfer data\n");
    }
}

/*
 * ============================================================================
 *   @func   srio_Create_Setup
 *
 *   @desc
 *     This routine setsup a structure with the required setup of the SRIO.
 * ============================================================================
*/
void srio_Create_Setup (
    CSL_SrioHwSetup *pSetup,
	int bootcomplete

)
{
    Uint32 index;
    /* Peripheral enable */
    pSetup->perEn = 1;

    /* While in shutdown, put memories in sleep mode */
    pSetup->periCntlSetup.swMemSleepOverride = 1;
    
    /* Enable loopback operation */
    pSetup->periCntlSetup.loopback = 1;
    
    /* Boot process is over (complete) */
    pSetup->periCntlSetup.bootComplete = 1;
    
    /* Set internal bus priority to 1 (next to lowest) */
    pSetup->periCntlSetup.busTransPriority = CSL_SRIO_BUS_TRANS_PRIORITY_1;
    
    /* UDI buffers are port based not proirity based */
	pSetup->periCntlSetup.bufferMode = CSL_SRIO_1X_MODE_PRIORITY;
    
    /* VBUS clock is of 333 MHz  from TPCC document section 2.3 */
    pSetup->periCntlSetup.prescalar = CSL_SRIO_CLK_PRESCALE_6;
    
    /* Enable port 0 PLL */
	pSetup->periCntlSetup.pllEn = 0xf; 
    /* Enable clocks to all domains */
    pSetup->gblEn = 1;

    /* Enable clock in each domain */
    for (index=0; index<9; index++) { /* 9 domains */
        pSetup->blkEn[index] = 1;    /* Enable each of it */
    }

    /* 8-bit id is 0xAB and 16-bit id is 0xBEEF */
    pSetup->deviceId1 = SRIO_SET_DEVICE_ID(SMALL_DEV_ID, LARGE_DEV_ID);

    /* 8-bit id is 0xAB and 16-bit id is 0xBEEF for multi-cast*/
    pSetup->deviceId2 = SRIO_SET_DEVICE_ID(SMALL_DEV_ID, LARGE_DEV_ID);

	/* configure the SERDES registers */
    
  	/* SERDES PLL configuration  */
	pSetup->serDesPllCfg.pllEnable = TRUE;
	//pSetup->serDesPllCfg.pllMplyFactor = CSL_SRIO_SERDES_PLL_MPLY_BY_6;
	pSetup->serDesPllCfg.pllMplyFactor = CSL_SRIO_SERDES_PLL_MPLY_BY_12_5;

    /* SERDES RX channel 0 enable */
	pSetup->serDesRxChannelCfg[0].enRx = TRUE; 
    pSetup->serDesRxChannelCfg[0].symAlign = CSL_SRIO_SERDES_SYM_ALIGN_COMMA; 
    pSetup->serDesRxChannelCfg[0].los = CSL_SRIO_SERDES_LOS_DET_DISABLE;
    pSetup->serDesRxChannelCfg[0].clockDataRecovery = 0x00;  /* first order */
    pSetup->serDesRxChannelCfg[0].equalizer = 0x01;
    pSetup->serDesRxChannelCfg[0].rate = CSL_SRIO_SERDES_RATE_FULL;
    
    /* SERDES TX channel 0 enable */
    pSetup->serDesTxChannelCfg[0].enTx = TRUE; 
    pSetup->serDesTxChannelCfg[0].enableFixedPhase = TRUE; 
    pSetup->serDesTxChannelCfg[0].commonMode = CSL_SRIO_SERDES_COMMON_MODE_RAISED; 
    pSetup->serDesTxChannelCfg[0].outputSwing = CSL_SRIO_SERDES_SWING_AMPLITUDE_1000; 
    pSetup->serDesTxChannelCfg[0].rate = CSL_SRIO_SERDES_RATE_FULL; 

    
    /* SERDES RX channel 1 enable */
	pSetup->serDesRxChannelCfg[1].enRx = TRUE; 
    pSetup->serDesRxChannelCfg[1].symAlign = CSL_SRIO_SERDES_SYM_ALIGN_COMMA; 
    pSetup->serDesRxChannelCfg[1].los = CSL_SRIO_SERDES_LOS_DET_DISABLE;
    pSetup->serDesRxChannelCfg[1].clockDataRecovery = 0x00;  /* first order */
    pSetup->serDesRxChannelCfg[1].equalizer = 0x01;
    pSetup->serDesRxChannelCfg[1].rate = CSL_SRIO_SERDES_RATE_FULL;

    /* SERDES TX channel 1 enable */
    pSetup->serDesTxChannelCfg[1].enTx = TRUE; 
    pSetup->serDesTxChannelCfg[1].enableFixedPhase = TRUE; 
    pSetup->serDesTxChannelCfg[1].commonMode = CSL_SRIO_SERDES_COMMON_MODE_RAISED; 
    pSetup->serDesTxChannelCfg[1].outputSwing = CSL_SRIO_SERDES_SWING_AMPLITUDE_1000; 
    pSetup->serDesTxChannelCfg[1].rate = CSL_SRIO_SERDES_RATE_FULL; 


    /* SERDES RX channel 2 enable */
	pSetup->serDesRxChannelCfg[2].enRx = TRUE; 
    pSetup->serDesRxChannelCfg[2].symAlign = CSL_SRIO_SERDES_SYM_ALIGN_COMMA; 
    pSetup->serDesRxChannelCfg[2].los = CSL_SRIO_SERDES_LOS_DET_DISABLE;
    pSetup->serDesRxChannelCfg[2].clockDataRecovery = 0x00;  /* first order */
    pSetup->serDesRxChannelCfg[2].equalizer = 0x01;
    pSetup->serDesRxChannelCfg[2].rate = CSL_SRIO_SERDES_RATE_FULL;

    /* SERDES TX channel 2 enable */
    pSetup->serDesTxChannelCfg[2].enTx = TRUE; 
    pSetup->serDesTxChannelCfg[2].enableFixedPhase = TRUE; 
    pSetup->serDesTxChannelCfg[2].commonMode = CSL_SRIO_SERDES_COMMON_MODE_RAISED; 
    pSetup->serDesTxChannelCfg[2].outputSwing = CSL_SRIO_SERDES_SWING_AMPLITUDE_1000; 
    pSetup->serDesTxChannelCfg[2].rate = CSL_SRIO_SERDES_RATE_FULL; 


    /* SERDES RX channel 3 enable */
	pSetup->serDesRxChannelCfg[3].enRx = TRUE; 
    pSetup->serDesRxChannelCfg[3].symAlign = CSL_SRIO_SERDES_SYM_ALIGN_COMMA; 
    pSetup->serDesRxChannelCfg[3].los = CSL_SRIO_SERDES_LOS_DET_DISABLE;
    pSetup->serDesRxChannelCfg[3].clockDataRecovery = 0x00;  /* first order */
    pSetup->serDesRxChannelCfg[3].equalizer = 0x01;
    pSetup->serDesRxChannelCfg[3].rate = CSL_SRIO_SERDES_RATE_FULL;

    /* SERDES TX channel 3 enable */
    pSetup->serDesTxChannelCfg[3].enTx = TRUE; 
    pSetup->serDesTxChannelCfg[3].enableFixedPhase = TRUE; 
    pSetup->serDesTxChannelCfg[3].commonMode = CSL_SRIO_SERDES_COMMON_MODE_RAISED; 
    pSetup->serDesTxChannelCfg[3].outputSwing = CSL_SRIO_SERDES_SWING_AMPLITUDE_1000; 
    pSetup->serDesTxChannelCfg[3].rate = CSL_SRIO_SERDES_RATE_FULL; 

    /* Select flow control ID length 16-bit */
    pSetup->flowCntlIdLen[0] = 1; 


    /* Destination ID of flow n, same ids as we are doing loopback */
    pSetup->flowCntlId[0] = LARGE_DEV_ID; 

    /* Sets the number of address bits generated by the PE as a source and 
     * processed by the PE as the target of an operation as 34 bits
     */
    pSetup->peLlAddrCtrl = CSL_SRIO_ADDR_SELECT_34BIT;

    /* Base device configuration */
    pSetup->devIdSetup.smallTrBaseDevId =  SMALL_DEV_ID;
    pSetup->devIdSetup.largeTrBaseDevId =  LARGE_DEV_ID;
    pSetup->devIdSetup.hostBaseDevId =  LARGE_DEV_ID;

    /* Port General configuration */
    pSetup->portGenSetup.portLinkTimeout = 0xFFFFF;  /* 215 ms */
    pSetup->portGenSetup.portRespTimeout = 0xFFFFF;  /* 215 ms */
    pSetup->portGenSetup.hostEn = 1;                 /* It is a slave */
    pSetup->portGenSetup.masterEn = 1;     /* This device can issue requests */

    /* Port control configuration */
    pSetup->portCntlSetup[0].portDis = 0;        /* Do not disable Port 0 */
    pSetup->portCntlSetup[0].outPortEn = 1;      /* Output on Port 0 enabled */
    pSetup->portCntlSetup[0].inPortEn = 1;       /* Input on Port 0 enabled */
    pSetup->portCntlSetup[0].portWidthOverride =
                CSL_SRIO_PORT_WIDTH_NO_OVERRIDE; /* 4 line port */
    pSetup->portCntlSetup[0].errCheckDis = 0;    /* Err check enabled */
    pSetup->portCntlSetup[0].multicastRcvEn = 0; /* MltCast receive enabled */
    pSetup->portCntlSetup[0].stopOnPortFailEn = 1; /* Stop on fail */
    pSetup->portCntlSetup[0].dropPktEn = 1;      /* Drop PKT */
    pSetup->portCntlSetup[0].portLockoutEn = 0;  /* Send any PKT */
    
    /* Enable all logical/transport errors */
    pSetup->lgclTransErrEn = CSL_SRIO_IO_ERR_RESP_ENABLE |
                             CSL_SRIO_ILL_TRANS_DECODE_ENABLE |
                             CSL_SRIO_ILL_TRANS_TARGET_ERR_ENABLE |
                             CSL_SRIO_PKT_RESP_TIMEOUT_ENABLE |
                             CSL_SRIO_UNSOLICITED_RESP_ENABLE |
                             CSL_SRIO_UNSUPPORTED_TRANS_ENABLE;

    /* Enable all Port errors */
    
    pSetup->portErrSetup[0].portErrRateEn =
                            CSL_SRIO_ERR_IMP_SPECIFIC_ENABLE |
                            CSL_SRIO_CORRUPT_CNTL_SYM_ENABLE |
                            CSL_SRIO_CNTL_SYM_UNEXPECTED_ACKID_ENABLE |
                            CSL_SRIO_RCVD_PKT_NOT_ACCPT_ENABLE |
                            CSL_SRIO_PKT_UNEXPECTED_ACKID_ENABLE |
                            CSL_SRIO_RCVD_PKT_WITH_BAD_CRC_ENABLE |
                            CSL_SRIO_RCVD_PKT_OVER_276B_ENABLE |
                            CSL_SRIO_NON_OUTSTANDING_ACKID_ENABLE |
                            CSL_SRIO_PROTOCOL_ERROR_ENABLE |
                            CSL_SRIO_UNSOLICITED_ACK_CNTL_SYM_ENABLE |
                            CSL_SRIO_LINK_TIMEOUT_ENABLE;

    /* Decrement error rate counter every second */ 
    pSetup->portErrSetup[0].prtErrRtBias = CSL_SRIO_ERR_RATE_BIAS_1S;

    /* Allow only 2 errors after error threshold is reached */ 
    pSetup->portErrSetup[0].portErrRtRec = CSL_SRIO_ERR_RATE_COUNT_2;

    /*  Port error setup */ 
    pSetup->portErrSetup[0].portErrRtFldThresh = 10;   /* Err threshold = 10 */
    pSetup->portErrSetup[0].portErrRtDegrdThresh = 10; /* Err degrade
                                                          threshold = 10 */
    
    
    /* This configures the SP_IP_MODE register */
    /*
       IDLE_ERR_DIS    - 0b0;  IDLE Error checking enabled
       TX_FIFO_BYP ASS - 0b0;  The TX_FIFO is operational
       PW_DIS          - 0b0;  Port-Write Error reporting is enabled
       TGT_ID_DIS      - 0b0;  packet accepted if DestID != BaseID
       SELF_RST        - 0b0;  Self reset disabled
       MLTC_EN         - 0b1;  Multicast-Event Interrupt Enable 
       RST_EN          - 0b1;  Reset Interrupt Enable
       PW_EN           - 0b1;  Port-Write-In Interrupt Enable

       Not writing into clear bits, assuming they are cleared on reset
    */
	pSetup->portIpModeSet = 0x0800003F;

    /* Configure the SP_IP_PRESCALE register assuming 333 MHz frequency */
    pSetup->portIpPrescalar = 33; 

    /*  Port-Write Timer. The timer defines a period to repeat sending an error
     *  reporting Port-Write request for software assistance. The timer stopped 
     *  by software writing to the error detect registers 900 ms
     */
    pSetup->pwTimer = CSL_SRIO_PW_TIME_8;

    /* Port control independent error reporting enable. Macros can be ORed
     *  to get the value
     */

    /*
       TX_FLW        - 0;     Receive link flow control
       SOFT_REC      - 0;     Hardware controlled error recovery 
       FORCE_REINIT  - 0;     Reinitialization process NOT forced
       TRANS_MODE    - 01;    transfer mode - Store & Forward Mode 
       DEBUG         - 1;     Debug enable
       SEND_DBG_PKT  - 0;     Do not force a debug packet
       ILL_TRANS_EN  - 1;     Illegal Transfer Error reporting Enable
       MAX_RETRY_EN  - 1;     Max_retry_error report enable
       MAX_RETRY_THR - 0x01;  Maximum Retry Threshold Trigger
       IRQ_EN        - 1;     Interrupt error report Enable
    */ 
    //pSetup->portCntlIndpEn[0] = 0x01A20180; 
     pSetup->portCntlIndpEn[0] = 0x01A20380; 
     pSetup->portCntlIndpEn[1] = 0x01A20380;
     pSetup->portCntlIndpEn[2] = 0x01A20380;
     pSetup->portCntlIndpEn[3] = 0x01A20380;
    
}

void enableSRIO (
)
{
    CSL_PscObj pscObj;
    CSL_PscHandle hPsc;
    CSL_Status status;
    CSL_PscPowerDomain pwrDmn = CSL_PSC_PWRDMN_PROXYA;
    CSL_PscPeripherals module = CSL_PSC_MODULE_SRIO;
    CSL_PscPwrDmnTransState response;

    memset(&pscObj, 0, sizeof(CSL_PscObj));      

    /* Init PSC module */
    CSL_pscInit(NULL);

    /* Open PSC module to get a Handle */
    hPsc = CSL_pscOpen(&pscObj, CSL_PSC, NULL, &status);

    /* Enable power domain for SRIO */
    CSL_pscHwControl(hPsc, CSL_PSC_CMD_ENABLE_PWRDMN, &pwrDmn);

    /* Enable clock for the specified module SRIO */
    CSL_pscHwControl(hPsc, CSL_PSC_CMD_ENABLE_MODULE, &module);
	
    /* Enable SRIO Power domain GO transition */
    CSL_pscHwControl(hPsc, CSL_PSC_CMD_PWRDMN_TRNS, &pwrDmn);

    response.pwrDmn = CSL_PSC_PWRDMN_PROXYA;
    response.status = 0x0; 
    do{
        CSL_pscGetHwStatus(hPsc, CSL_PSC_QUERY_PWRDMN_TRANS_STAT, (void *)&response);
        (response.status) &= (1 << CSL_PSC_PWRDMN_PROXYA);
    }while((response.status) != 0x0);

    /* Close the PSC module */
    CSL_pscClose(hPsc);
}

