/*  ============================================================================
 *   Copyright (c) Texas Instruments Inc 2002, 2003, 2004, 2005, 2006, 2008
 *
 *   Use of this software is controlled by the terms and conditions found in the
 *   license agreement under which this software has been supplied.
 *   ===========================================================================
 */
/** ===========================================================================
 *   @file tetb_example.c
 *
 *   @desc This is a example of Application Managed On-Chip Capture of Trace.
 *
 *  ============================================================================
 *   @n <b> Example Description </b>
 *   @n This is an example of the CSL TETB.
 *   @verbatim
       1. Initialize DTF.
       2. Initialize TETB.
       3. Set trace clock i.e., cpu/3
       4. Claim DTF ownership
       5. Enable DTF ownership
       6. Claim ownership for trace export block
       7. Program Pin manager to have EMU pin allocation
       8. Enable ownership for trace export block to start generating trace test
          pattern.
       9. Disable formatting and put TETB formatter into bypass mode.
       10.Enable TETB data capture by writing TETB Control Register.
       11.Program trace control based on width
       12.Generate the specified trace test pattern
       13.change trace clock to cpu/6
       14.Disable TETB data capture by writing TETB Control Register.
       15.Initialize the TETB RAM read pointer register with start_addr
       16.Read 1KB trace data out of TETB and log it into L2 memory
       17.Re-nable TETB data capture by writing TETB Control Register.
       18.Release DTF ownership
       19.Release ownership for trace export block

     @endverbatim
 *
 *=============================================================================
 *
 *   <b> Procedure to run the example </b>
 *   @verbatim
      1. Configure the CCS setup
      2. Please refer CCS manual for setup configuration and loading
         proper GEL file
      3. Launch CCS window
      4. Open project Tetb_example.pjt
      5. Build the project and load the .out file of the project.

     @endverbatim
 *
 *
 * ============================================================================
 */

/* =============================================================================
 *  Revision History
 *  ================
 *  15-Oct-2008     Ported from ST
 *  29-Apr-2008		File Created for basic test of TETB module.
 * =============================================================================
 */
/****************************************************************************/
/* Include Files                                                            */
/****************************************************************************/
#include <csl_tetb.h>
#include <csl_adtf2.h>
#include <soc.h>
#include <stdio.h>
#include <csl_psc.h>

/****************************************************************************/
/* Defines                                                            */
/****************************************************************************/
// Size of TETB RAM in 32-bit words
#define TETB_RAM_SIZE    512

// Trace control register MMR address.

// Trace Acquisition
#define TR_CNTL_ADDR    0x01BC013C
#define TF_CNTL_ADDR    0x01BC014C
#define TR_SYNC_ADDR    0x01BC0140
#define TR_CNTL  	    *((volatile unsigned int*)TR_CNTL_ADDR) 
#define TF_CNTL  	    *((volatile unsigned int*)TF_CNTL_ADDR) 
#define TR_SYNC  	    *((volatile unsigned int*)TR_SYNC_ADDR)

// PIN manager MMR address. 
#define PIN_MGR0_ADDR	0x01BC0130
#define PIN_MGR1_ADDR	0x01BC0134
#define PIN_MGR2_ADDR	0x01BC0138
#define PIN_MGR3_ADDR	0x01BC0164
#define PIN_OBSV_ADDR	0x01BC0160

#define PIN_MGR0  	    *((volatile unsigned int*)PIN_MGR0_ADDR) 
#define PIN_MGR1  	    *((volatile unsigned int*)PIN_MGR1_ADDR) 
#define PIN_MGR2  	    *((volatile unsigned int*)PIN_MGR2_ADDR) 
#define PIN_MGR3  	    *((volatile unsigned int*)PIN_MGR3_ADDR) 
#define PIN_OBSV  	    *((volatile unsigned int*)PIN_OBSV_ADDR) 

#define TETB_UNLOCK     0xC5ACCE55 // Value to unlock TETB for register accesses
#define TETB_LOCK       0x00000000 // Value to unlock TETB for register accesses

// TETB base address
#define TETB_BASE  	    0x02AD0000 // Base Address for TETB MMRs associated with GEMx
#define TETB_FFSR_ADDR	(TETB_BASE + 0x300)
#define TETB_FFCR_ADDR	(TETB_BASE + 0x304)
#define TETB_IRST_ADDR  (TETB_BASE + 0xE00) // TETB Interrupt Raw Status Register
#define TETB_ICST_ADDR  (TETB_BASE + 0xE04) // TETB Interrupt Clear Shadow Register
#define TETB_ICST       *((volatile unsigned int*)TETB_ICST_ADDR) // TETB Interrupt Clear Shadow Register
#define TETB_IRST       *((volatile unsigned int*)TETB_IRST_ADDR) // TETB Interrupt Raw Status Register
#define TETB_FFSR  	    *((volatile unsigned int*)TETB_FFSR_ADDR) // TETB Formatter and Flush Status Register FFSR
#define TETB_FFCR  	    *((volatile unsigned int*)TETB_FFCR_ADDR) // TETB Formatter and Flush Control Register FFCR


/*  dbgtrace generates 49050 total number of data. */
#define NUM_TRACE_DATA   1024 

/****************************************************************************/     
/* Global Variables                                                         */     
/****************************************************************************/  

/* Object declaration */
CSL_TetbHandle   hTetb;
CSL_TetbObj      tetbObj;
CSL_Adtf2Handle  hAdtf2;
CSL_Adtf2Obj 	 adtf2Obj;
  
/*-----------------------------------------------------------------------------
 * Pragmas used to create special linker sections
 *----------------------------------------------------------------------------*/
/*  We need to know the mail_box variable address from host side testcase, hence 
 *  we link it into a separate section 
 *  Define a memory buffer to store the data read out of TETB RAM so that the host
 *  testcase can read out and log to a file. The app code cannot directly log data 
 *  into file since we don't have CIO support here as we do in the CCS debugger. 
 */

#pragma DATA_ALIGN(tetb_ram_data,64);
#pragma DATA_SECTION(tetb_ram_data,".srctestMemL2");

Uint32      tetb_ram_data[NUM_TRACE_DATA];
Uint32      *tetb_ram_dataptr;
Uint32      rec_data_20[NUM_TRACE_DATA];
Uint32      exp_data_20[NUM_TRACE_DATA]; 
Uint32      SizeRWP;
int         flow;
unsigned    pattern =1;

#pragma DATA_SECTION(start_time,".srctestMemL2");
#pragma DATA_SECTION(end_time,".srctestMemL2");
long long   start_time, end_time;

extern __cregister volatile Uint32 TCU_CNTL;

/****************************************************************************/     
/* Functions                                                                */     
/****************************************************************************/ 
void tetb_initialization (void);
void adtf2_initialization (void);
int config_gem_trace_pat(int pattern);
int verify_trace_pat_20_tetb(int pattern,Uint32* Addr, int size);
Uint32 tetb_example(void);
void enableADTF2(void);

void main(void)
{
   Uint32 result;
	 
    printf("ENTERING TETB FUNCTIONALITY TESTING....\n");

    result = tetb_example();

	if(result == 0)
    {
    	printf("TETB: Functionality example SUCCEEDED\n");
	}
	else
	{
		printf("TETB: Functionality example FAILED\n");
		printf("TETB: Number of mismatch is %d\n",result);
	}
}

/* =============================================================================
 *   @func   tetb_example
 *
 *   @desc
 *       This function initializes the ADTF2 and TETB modules. Calls the config_gem_trace_pat
 *       function to capture and verify the captured data 
 *
 *   @arg   None
 *
 *   @return 
 *      0  - successful
 *      -1 - ERROR
 *      >0 - number of mismatched patterns        
 * =============================================================================
 */

Uint32 tetb_example()
{
 
   Uint32       Result;

	enableADTF2();
    adtf2_initialization();

	/* Change ownership on the pins to debugger */

	TR_CNTL = 0x1;
	TR_CNTL = 0x2;
	TR_CNTL = 0xE;

    hAdtf2->regs->LAR=TETB_UNLOCK;

	hAdtf2->regs->ADTFCSR=0x5;

	/* Claim ownership for trace export block 
	Some how this allows you to set the PIN_MGR registers*/
	TCU_CNTL = 1;

    /* Program Pin manager to have EMU pin allocation as follows to conform to DTF spec 3.2.1 requirements 
     EMU0=trace_data[19], EMU1=trace_data[18], ..., EMU19=trace_data[0],  
     EMU20=clock0, EMU21=clock1                                           
    20 bits*/
    PIN_MGR0 = 0xCCDDEEFF;  
    PIN_MGR1 = 0x8899AABB; 
   	PIN_MGR2 = 0x00556677;
    PIN_MGR3 = 0x00000000;

    /* Enable ownership for trace export block to start generating trace test pattern. */
	TCU_CNTL = 2;

    /* Program trace control based on width */
    TR_CNTL = 0x0001C07D; // claim //

    /* Generate the specified trace test pattern */
    if (pattern == 0)
        flow = 8;
    else if (pattern == 1)
        flow = 9;
    else if (pattern == 2)
        flow = 10;
    else if (pattern == 3)
        flow = 11;

	Result = config_gem_trace_pat(pattern);

	return (Result);
}

/* =============================================================================
 *   @func   config_gem_trace_pat
 *
 *   @desc
 *       This function sets up the GEM Pin Manager, trace control/format registers
 * to generate the specified test pattern and use DTF/TETB to capture.       
 * This function also retrieve data from TETB and store them into L2 mem for 
 * host testcase to check.										
 *																			
 * Note: GEM always generate 20-bit test pattern regardless of in RUN_MAX   
 *       mode or RUN_WIDTH mode.											
 * Note: Current dbgtrace always record data into 10-bit packet format.		
 * Note: This verification function assume the trace data is captured from  
 *       the very beginning of pattern generation.							
 * Note: This function assumes at least 7 EMU pins (5 data +2 clk)          
 *       is available for off-chip trace.
 *
 *   @arg   
 *      pattern - Test pattern number (0~3)
 *
 *   @return 
 *      0  - successful
 *      -1 - ERROR
 *      >0 - number of mismatched patterns        
 * =============================================================================
 */
    
int config_gem_trace_pat(int pattern)
{
    Uint32          Result = 0;
	int             i=0;
	CSL_Status      status;
    Uint32          response=0;
	Uint32          arg;

	/* Clear all interrupts*/
	tetb_ram_dataptr = tetb_ram_data;
	SizeRWP = TETB_RAM_SIZE;
   
	for (i=0;i<512;i++)
	{
		tetb_ram_data[i] = 0;  // Trace Buffer Memory
	}
    
    tetb_initialization();

    /* Enable all interrupts*/
	arg = 0;
	status = CSL_tetbHwControl(hTetb, CSL_TETB_CMD_CLEAR_ACQ_COMP, &arg);
    if (status != CSL_SOK)
	{
		printf ("TETB: CSL_TETB_CMD_CLEAR_ACQ_COMP... Failed.\n");
		return -1;
	}

	status = CSL_tetbHwControl(hTetb, CSL_TETB_CMD_CLEAR_UNDERFLOW, &arg);
	if (status != CSL_SOK)
	{
		printf ("TETB: CSL_TETB_CMD_CLEAR_UNDERFLOW... Failed.\n");
		return -1;
	}

	status = CSL_tetbHwControl(hTetb, CSL_TETB_CMD_CLEAR_OVERFLOW, &arg);
	if (status != CSL_SOK)
	{
		printf ("TETB: CSL_TETB_CMD_CLEAR_OVERFLOW... Failed.\n");
		return -1;
	}

	status = CSL_tetbHwControl(hTetb, CSL_TETB_CMD_CLEAR_FULL, &arg);
	if (status != CSL_SOK)
	{
		printf ("TETB: CSL_TETB_CMD_CLEAR_FULL... Failed.\n");
		return -1;
	}
	status = CSL_tetbHwControl(hTetb, CSL_TETB_CMD_CLEAR_HALF_FULL, &arg);
	if (status != CSL_SOK)
	{
		printf ("TETB: CSL_TETB_CMD_CLEAR_HALF_FULL... Failed.\n");
		return -1;
	}

	arg = 0;
	status = CSL_tetbHwControl(hTetb, CSL_TETB_CMD_DISABLE_ACQ_COMP, &arg);
	if (status != CSL_SOK)
	{
		printf ("TETB: CSL_TETB_CMD_DISABLE_ACQ_COMP... Failed.\n");
		return -1;
	}

	status = CSL_tetbHwControl(hTetb, CSL_TETB_CMD_DISABLE_OVERFLOW, &arg);
	if (status != CSL_SOK)
	{
		printf ("TETB: CSL_TETB_CMD_DISABLE_OVERFLOW... Failed.\n");
		return -1;
	}

	status = CSL_tetbHwControl(hTetb, CSL_TETB_CMD_DISABLE_UNDERFLOW, &arg);
	if (status != CSL_SOK)
	{
		printf ("TETB: CSL_TETB_CMD_DISABLE_UNDERFLOW... Failed.\n");
		return -1;
	}

	status = CSL_tetbHwControl(hTetb, CSL_TETB_CMD_DISABLE_HALF_FULL, &arg);
	if (status != CSL_SOK)
	{
		printf ("TETB: CSL_TETB_CMD_DISABLE_HALF_FULL... Failed.\n");
		return -1;
	}

	status = CSL_tetbHwControl(hTetb, CSL_TETB_CMD_DISABLE_FULL, &arg);
	if (status != CSL_SOK)
	{
		printf ("TETB: CSL_TETB_CMD_DISABLE_FULL... Failed.\n");
		return -1;
	}

	/* Enable all interrupts*/
	arg = 0;
	status = CSL_tetbHwControl(hTetb, CSL_TETB_CMD_ENABLE_OVERFLOW, &arg);
	if (status != CSL_SOK)
	{
		printf ("TETB: CSL_TETB_CMD_ENABLE_OVERFLOW... Failed.\n");
		return -1;
	}

	status = CSL_tetbHwControl(hTetb, CSL_TETB_CMD_ENABLE_UNDERFLOW, &arg);
	if (status != CSL_SOK)
	{
		printf ("TETB: CSL_TETB_CMD_ENABLE_UNDERFLOW... Failed.\n");
		return -1;
	}

	status = CSL_tetbHwControl(hTetb, CSL_TETB_CMD_ENABLE_HALF_FULL, &arg);
	if (status != CSL_SOK)
	{
		printf ("TETB: CSL_TETB_CMD_ENABLE_HALF_FULL... Failed.\n");
		return -1;
	}
 
    status = CSL_tetbHwControl(hTetb, CSL_TETB_CMD_ENABLE_FULL, &arg);
	if (status != CSL_SOK)
	{
		printf ("TETB: CSL_TETB_CMD_ENABLE_FULL... Failed.\n");
		return -1;
	}

	hTetb->regs->TCTL = 0x0000; /* TETB FIFO reset by writing 0 to TETB RAM Write Pointer Register. */ 
    arg = 0;
    status = CSL_tetbHwControl(hTetb, CSL_TETB_CMD_SET_RAM_WR_POINTER, &arg);
	if (status != CSL_SOK)
	{
		printf ("TETB: CSL_TETB_CMD_SET_RAM_WR_POINTER... Failed.\n");
		return -1;
	}

    arg = 0;
    status = CSL_tetbHwControl(hTetb, CSL_TETB_CMD_SET_RAM_RD_POINTER, &arg);
	if (status != CSL_SOK)
	{
		printf ("TETB: CSL_TETB_CMD_SET_RAM_RD_POINTER... Failed.\n");
		return -1;
	}

	/*??????????????????????????????? Not in the spec
     Disable formatting and put TETB formatter into bypass mode. */
    TETB_FFCR = 0; /* EnFCont=0, EnFTC=0 */

	while (TETB_IRST != 0x00)
	{
		arg = 0;
    	status = CSL_tetbHwControl(hTetb, CSL_TETB_CMD_CLEAR_ACQ_COMP, &arg);
	    if (status != CSL_SOK)
    	{
	    	printf ("TETB: CSL_TETB_CMD_CLEAR_ACQ_COMP... Failed.\n");
		    return -1;
    	}
	
	    status = CSL_tetbHwControl(hTetb, CSL_TETB_CMD_CLEAR_UNDERFLOW, &arg);
    	if (status != CSL_SOK)
	    {
		    printf ("TETB: CSL_TETB_CMD_CLEAR_UNDERFLOW... Failed.\n");
    		return -1;
	    }

    	status = CSL_tetbHwControl(hTetb, CSL_TETB_CMD_CLEAR_OVERFLOW, &arg);
	    if (status != CSL_SOK)
    	{
		    printf ("TETB: CSL_TETB_CMD_CLEAR_OVERFLOW... Failed.\n");
	    	return -1;
    	}

	    status = CSL_tetbHwControl(hTetb, CSL_TETB_CMD_CLEAR_FULL, &arg);
    	if (status != CSL_SOK)
	    {
		    printf ("TETB: CSL_TETB_CMD_CLEAR_FULL... Failed.\n");
    		return -1;
	    }

    	status = CSL_tetbHwControl(hTetb, CSL_TETB_CMD_CLEAR_HALF_FULL, &arg);
	    if (status != CSL_SOK)
    	{
	    	printf ("TETB: CSL_TETB_CMD_CLEAR_HALF_FULL... Failed.\n");
		    return -1;
    	}

    }

    /* Enable TETB data capture by writing TETB Control Register. */
    status = CSL_tetbHwControl(hTetb, CSL_TETB_CMD_ENA_TRACE_CAPTURE, NULL);

	do {
        status = CSL_tetbGetHwStatus(hTetb, CSL_TETB_QUERY_TRACECAP_STATUS,
                                    &response);
		if (status != CSL_SOK)
		{
			printf ("TETB: CSL_TETB_QUERY_TRACECAP_STATUS... Failed.\n");
			return -1;
		}

	} while (response != CSL_TETB_CTL_TRACECAPTEN_ENABLE);

	/* Start the Packet Transmitter */
    TF_CNTL = (0x1400) | (flow) | 0x20; // TF_CNTL: width=20, flow=specified, Go=1, Active=0 
    
    /* Poll for Half Full */
	while(hTetb->regs->RWP < TETB_RAM_SIZE);

    /* Halt the Packet Transmitter */
    TF_CNTL = (0x1400) | (flow); // TF_CNTL: width=20, flow=specified, Go=1, Active=0 

	/* Flush Trace Formatter */ 
    TF_CNTL = 0x00008000;

	TETB_ICST=0x0; //Clear Inetrrupt Status

    /* Disable TETB data capture by writing TETB Control Register. */
    status = CSL_tetbHwControl(hTetb, CSL_TETB_CMD_DIS_TRACE_CAPTURE, NULL);
	if (status != CSL_SOK)
	{
		printf ("TETB: CSL_TETB_CMD_DIS_TRACE_CAPTURE... Failed.\n");
		return -1;
	}

	do {
        status = CSL_tetbGetHwStatus(hTetb, CSL_TETB_QUERY_TRACECAP_STATUS,
                                    &response);
		if (status != CSL_SOK)
		{
			printf ("TETB: CSL_TETB_QUERY_TRACECAP_STATUS... Failed.\n");
			return -1;
		}

	} while (response != CSL_TETB_CTL_TRACECAPTEN_DISABLE);

	/* Read */
	for (i = 0; i < TETB_RAM_SIZE; i++)
		status = CSL_tetbRead(hTetb, &tetb_ram_data[i]);

    printf("Read trace data from the RAM read data register into tetb_ram_data buffer \n");

	Result = verify_trace_pat_20_tetb(pattern, tetb_ram_data, TETB_RAM_SIZE);

    return Result;
}

/* =============================================================================
 *   @func   verify_trace_pat_20_tetb
 *
 *   @desc
 *      This function also verifies the GEM trace test pattern recorded     
 *      into TETB (without NOP packet)										
 *																			
 * Note: GEM always generate 20-bit test pattern regardless of in RUN_MAX   
 *       mode or RUN_WIDTH mode.											
 * Note: Current dbgtrace always record data into 10-bit packet format.		
 * Note: This verification function assume the trace data is captured from  
 *       the very beginning of pattern generation.							
 * Note: This function assumes at least 7 EMU pins (5 data +2 clk)          
 *       is available for off-chip trace.
 *
 *   @arg   
 *      pattern - pattern to verify
 *      Addr - RAM address where data is stored
 *      size - size of data in RAM
 *
 *   @return 
 *      0  - if successful
 *      >0 - number of mismatched patterns        
 * =============================================================================
 */
    
int verify_trace_pat_20_tetb(int pattern, Uint32 *Addr, int size)
{
	int     i=0, j=0, x;
	int     error =0;
	Uint32  temp=0;
	int     sizecompare;


	/*************************************************************************************
	* Now extracted the 20-bit trace data out of the log file as recorded by dbgtrace utility. 
	*************************************************************************************/
	while((Addr[j] == 0xC0000000) || (Addr[j] == 0x00000000))
    	j++;


	for(x=0; x <= size; x++, j++)
	{
    	temp = Addr[j];
		rec_data_20[i++] = temp & 0x3FFFFFFF;
	}

	/*************************************************************************************
	* Now Calculate the expected 20-bit trace data based on test pattern
	*************************************************************************************/

	for(i=0;i<=size;i+=42)
	{
		// Beginning pattern 
		exp_data_20[i+0] = 0x3FEFFFFF;
		exp_data_20[i+2] = 0x3FDFFFFE;
		exp_data_20[i+4] = 0x3FBFFFFD;
		exp_data_20[i+6] = 0x3F7FFFFB;
		exp_data_20[i+8] = 0x3EFFFFF7;
		exp_data_20[i+10] = 0x3DFFFFEF;
		exp_data_20[i+12] = 0x3BFFFFDF;
		exp_data_20[i+14] = 0x37FFFFBF;
		exp_data_20[i+16] = 0x2FFFFF7F;
		exp_data_20[i+18] = 0x1FFFFEFF;
		exp_data_20[i+20] = 0x3FFFFDFF;
		exp_data_20[i+22] = 0x3FFFFBFF;
		exp_data_20[i+24] = 0x3FFFF7FF;
		exp_data_20[i+26] = 0x3FFFEFFF;
		exp_data_20[i+28] = 0x3FFFDFFF;
		exp_data_20[i+30] = 0x3FFFBFFF;
		exp_data_20[i+32] = 0x3FFF7FFF;
		exp_data_20[i+34] = 0x3FFEFFFF;
		exp_data_20[i+36] = 0x3FFDFFFF;
		exp_data_20[i+38] = 0x3FFBFFFF;
		exp_data_20[i+40] = 0x3FF7FFFF;
	
		exp_data_20[i+1] = 0x000007FF;
		exp_data_20[i+3] = 0x00000BFF;
		exp_data_20[i+5] = 0x000013FF;
		exp_data_20[i+7] = 0x000023FF;
		exp_data_20[i+9] = 0x000043FF;
		exp_data_20[i+11] = 0x000083FF;
		exp_data_20[i+13] = 0x000103FF;
		exp_data_20[i+15] = 0x000203FF;
		exp_data_20[i+17] = 0x000403FF;
		exp_data_20[i+19] = 0x000803FF;
		exp_data_20[i+21] = 0x001003FE;
		exp_data_20[i+23] = 0x002003FD;
		exp_data_20[i+25] = 0x004003FB;
		exp_data_20[i+27] = 0x008003F7;
		exp_data_20[i+29] = 0x010003EF;
		exp_data_20[i+31] = 0x020003DF;
		exp_data_20[i+33] = 0x040003BF;
		exp_data_20[i+35] = 0x0800037F;
		exp_data_20[i+37] = 0x100002FF;
		exp_data_20[i+39] = 0x200001FF;
		exp_data_20[i+41] = 0x000003FF;		
	
	}
	sizecompare = i-42;
	// ************************************************************************************
	//  Now verify the received trace data against the expected 10-bit trace data
	// *********************************************************************************** 
	for (i=0; i< sizecompare; i++)
	{
		if (rec_data_20[i] != exp_data_20[i])
			error++;
	}

	return error;
}

/* =============================================================================
 *   @func   trace_clk_on
 *
 *   @desc
 *      This function changes the trace clock frequency.
 *
 *   @arg    None
 *
 *   @return 
 *      -1 - ERROR if div_val > 32
 *      0  - if successful
 *
 * =============================================================================
 */
int trace_clk_on(unsigned int div_val)
{
	//  turn on trace clock from PLL here.  // 
	int plldiv_addr;
	int pllcmd_addr;
	int pllcmd_val;
    int plldiv_val;

	pllcmd_addr = 0x029A0138; 
    plldiv_addr = 0x029A0178;
    
	/*  PLLDIV register value: 
	bit 4:0 -- ratio = 00011b (/4), 11111b (/32)
	bit 15 -- DnEn = 1  */ 
	//  PLL divider value could be from 1 to 32. */
	if (div_val > 32)
		return -1;

	plldiv_val = (1<<15) | (div_val -1);
	*(unsigned int*)plldiv_addr = plldiv_val;

	/*  PLLCMD register value:
	bit 1 -- GOSET */
	pllcmd_val = 0x1;
	*(unsigned int*)pllcmd_addr = pllcmd_val;

    return 0;

}

/* =============================================================================
 *   @func   tetb_initialization
 *
 *   @desc
 *      This function initilaizes tetb and opens specified instance of tetb.
 *
 *   @arg    None
 *
 *   @return None
 *
 * =============================================================================
 */
void tetb_initialization (void)
{
    CSL_Status     status;

    /* Initialize  TETB CSL module */
    status = CSL_tetbInit(NULL);
    if (status != CSL_SOK) 
    {
        printf("TETB: Initialization... Failed.\n");
        printf("\tReason: CSL_tetbInit failed. [status = %d].\n", status);
        return;
    }
    else
        printf("TETB: Initialization... Passed.\n");

    /* Open TETB instance and verify */

    memset(&tetbObj, 0, sizeof(CSL_TetbObj));

    /* Open the CSL module for instance 0 */
    hTetb = CSL_tetbOpen(&tetbObj, (CSL_InstNum)CSL_TETB, NULL, &status);

    if ((hTetb == NULL) || (status != CSL_SOK)) 
    {
        printf("TETB: Opening instance0... Failed.\n");
        printf("\tReason: Error opening the instance0. \
                [status = 0x%x, hTetb = 0x%x]\n", status, hTetb);
        return;
    }
}



/* =============================================================================
 *   @func   adtf2_initialization
 *
 *   @desc
 *      This function initilaizes tetb and opens specified instance of dtf.
 *
 *   @arg    None
 *
 *   @return None
 *
 * =============================================================================
 */
void adtf2_initialization (void)
{
    CSL_Status     status;

    /* Initialize  DTF CSL module */
    status = CSL_adtf2Init(NULL);
    if (status != CSL_SOK) 
    {
        printf("ADTF2: Initialization... Failed.\n");
        printf("\tReason: CSL_adtf2Init failed. [status = %d].\n", status);
        return;
    }
    else 
        printf("ADTF2: Initialization... Passed.\n");

    /* Open ADTF2 instance and verify */

    memset(&adtf2Obj, 0, sizeof(CSL_Adtf2Obj));

    /* Open the CSL module for instance 0 */
    hAdtf2 = CSL_adtf2Open(&adtf2Obj, (CSL_InstNum)CSL_ADTF2_0, NULL, &status);

    if ((hAdtf2 == NULL) || (status != CSL_SOK)) 
    {
        printf("ADTF2: Opening instance0... Failed.\n");
        printf("\tReason: Error opening the instance0. \
                [status = 0x%x, hDtf = 0x%x]\n", status, hAdtf2);
        return;
    }
}

/* =============================================================================
 *   @func   enableADTF2
 *
 *   @desc
 *      This function powers ADTF2 module.
 *
 *   @arg    None
 *
 *   @return None
 *
 * =============================================================================
 */

void enableADTF2(void)
{
    CSL_PscObj pscObj;
    CSL_PscHandle hPsc;
    CSL_Status status;
    CSL_PscPowerDomain pwrDmn = CSL_PSC_PWRDMN_ALWAYSON;
    CSL_PscPeripherals module = CSL_PSC_MODULE_ADTF2;
    CSL_PscPwrDmnTransState response;

    memset(&pscObj, 0, sizeof(CSL_PscObj));      

    /* Init PSC module */
    CSL_pscInit(NULL);

    /* Open PSC module to get a Handle */
    hPsc = CSL_pscOpen(&pscObj, CSL_PSC, NULL, &status);

    /* Enable power domain for ADTF2 */
    CSL_pscHwControl(hPsc, CSL_PSC_CMD_ENABLE_PWRDMN, &pwrDmn);

    /* Enable clock for the specified module ADTF2 */
    CSL_pscHwControl(hPsc, CSL_PSC_CMD_ENABLE_MODULE, &module);
	
    /* Enable ADTF2 Power domain GO transition */
    CSL_pscHwControl(hPsc, CSL_PSC_CMD_PWRDMN_TRNS, &pwrDmn);

    response.pwrDmn = CSL_PSC_PWRDMN_ALWAYSON;
    response.status = 0x0; 
    do{
        CSL_pscGetHwStatus(hPsc, CSL_PSC_QUERY_PWRDMN_TRANS_STAT, (void *)&response);
        (response.status) &= (1 << CSL_PSC_PWRDMN_ALWAYSON);
    }while((response.status) != 0x0);

    /* Close the PSC module */
    CSL_pscClose(hPsc);
}

