/*  ============================================================================
 * Copyright (c) Texas Instruments Inc 2002, 2003, 2004, 2005, 2006, 2007, 2008
 *
 *   Use of this software is controlled by the terms and conditions found in the
 *   license agreement under which this software has been supplied.
 *  ============================================================================
 */

/** ============================================================================
 *   @file  csl_cacheL1.c
 *
 *   @path  $(CSLPATH)\src\cache
 *
 *   @desc  File for functional layer of L1 Cache
 *
 */
 
/* =============================================================================
 *  Revision History
 *  ===============
 *  23-Mar-2004 Jamon Bowen File Created
 *
 *  21-Jun-2004 Ruchika Kharwar modified.
 *
 *  24-Jan-2006 ds Updated CACHE_setL1pSize() API to return Old value  
 *  08-Sep-2008 Update for Doxygen
 * ============================================================================= 
 */

#include <csl_cache.h>
#include <_csl_cache.h>

/* ============================================================================
 *    CACHE_freezeL1
 *  ============================================================================
 */
#pragma CODE_SECTION (CACHE_freezeL1, ".text:csl_section:cache");
CACHE_L1_Freeze CACHE_freezeL1(void)
{
    Uint32 oldL1DFrz;
    Uint32 oldL1PFrz;
    Uint32 restore_value;

    /* Critical section */
    restore_value = _disable_interrupts();

    ((CSL_CacheRegsOvly)CSL_CACHE_0_REGS)->L1DCC = \
                        CSL_FMK(CACHE_L1DCC_OPER, CSL_CACHE_L1DCC_OPER_FREEZE);
    ((CSL_CacheRegsOvly)CSL_CACHE_0_REGS)->L1PCC = \
                        CSL_FMK(CACHE_L1PCC_OPER, CSL_CACHE_L1PCC_OPER_FREEZE);
    oldL1DFrz = CSL_FEXT(((CSL_CacheRegsOvly) \
                                  CSL_CACHE_0_REGS)->L1DCC, CACHE_L1DCC_POPER);
    oldL1PFrz = CSL_FEXT(((CSL_CacheRegsOvly) \
                                  CSL_CACHE_0_REGS)->L1PCC, CACHE_L1PCC_POPER);

    _restore_interrupts(restore_value);
    /* End Critical section */

    /*  return cache state if a single freeze is returned then the other */
    if (oldL1DFrz == CSL_CACHE_L1DCC_OPER_FREEZE) {
        if (oldL1PFrz == CSL_CACHE_L1PCC_OPER_FREEZE)
            return CACHE_L1_FREEZE;
        else
            return CACHE_L1D_FREEZE;    
    }
    else {
        if (oldL1PFrz == CSL_CACHE_L1PCC_OPER_FREEZE)
            return CACHE_L1P_FREEZE;
        else
            return CACHE_L1_NORMAL;
    }
}

/* ============================================================================
 *    CACHE_unfreezeL1
 *  ============================================================================
 */
#pragma CODE_SECTION (CACHE_unfreezeL1, ".text:csl_section:cache");
CACHE_L1_Freeze CACHE_unfreezeL1 (void)
{
    Uint32      oldL1DFrz;
    Uint32      oldL1PFrz;
    Uint32      restore_value;
      
    /* Critical section */
    restore_value = _disable_interrupts();
    
    ((CSL_CacheRegsOvly)CSL_CACHE_0_REGS)->L1DCC = \
                          CSL_FMK(CACHE_L1DCC_OPER, CSL_CACHE_L1DCC_OPER_NORM);
    ((CSL_CacheRegsOvly)CSL_CACHE_0_REGS)->L1PCC = \
                          CSL_FMK(CACHE_L1PCC_OPER, CSL_CACHE_L1PCC_OPER_NORM);
    oldL1DFrz = CSL_FEXT(((CSL_CacheRegsOvly) \
                                  CSL_CACHE_0_REGS)->L1DCC, CACHE_L1DCC_POPER);
    oldL1PFrz = CSL_FEXT(((CSL_CacheRegsOvly) \
                                  CSL_CACHE_0_REGS)->L1PCC, CACHE_L1PCC_POPER);
    
    /* End Critical section */
    _restore_interrupts(restore_value);
    
    /* return cache state if a single freeze is returned then the other
     * cache is in normal mode
     */
    if (oldL1DFrz == CSL_CACHE_L1DCC_OPER_FREEZE) {
        if (oldL1PFrz == CSL_CACHE_L1PCC_OPER_FREEZE)
            return CACHE_L1_FREEZE;
        else
            return CACHE_L1D_FREEZE;    
    } 
    else {
        if (oldL1PFrz == CSL_CACHE_L1PCC_OPER_FREEZE)
            return CACHE_L1P_FREEZE;
        else
            return CACHE_L1_NORMAL;
    }
}

/* ============================================================================
 *    CACHE_setL1pSize
 *  ============================================================================
 */
#pragma CODE_SECTION (CACHE_setL1pSize, ".text:csl_section:cache");
CACHE_L1Size CACHE_setL1pSize (
    CACHE_L1Size    newSize
)
{
    Uint32      curSize;
    Uint32      restore_value;
     
    curSize = ((CSL_CacheRegsOvly)CSL_CACHE_0_REGS)->L1PCFG;
    
    /* Critical section */
    restore_value = _disable_interrupts();
    
    CSL_FINS(((CSL_CacheRegsOvly)CSL_CACHE_0_REGS)->L1PCFG, CACHE_L1PCFG_MODE, newSize);
    newSize = (CACHE_L1Size)((CSL_CacheRegsOvly)CSL_CACHE_0_REGS)->L1PCFG;
    
    /* End Critical section */
    _restore_interrupts(restore_value);
        
    return (CACHE_L1Size) (curSize);
}

/* ============================================================================
 *    CACHE_freezeL1p
 *  ============================================================================
 */
#pragma CODE_SECTION (CACHE_freezeL1p, ".text:csl_section:cache");
CACHE_L1_Freeze CACHE_freezeL1p (void)
{
    Uint32      oldL1PFrz;
    Uint32      restore_value;
     
    /* Critical section */
    restore_value = _disable_interrupts();
    
    ((CSL_CacheRegsOvly)CSL_CACHE_0_REGS)->L1PCC =  \
                        CSL_FMK(CACHE_L1PCC_OPER, CSL_CACHE_L1PCC_OPER_FREEZE);
    oldL1PFrz = 
        CSL_FEXT(((CSL_CacheRegsOvly) \
                                  CSL_CACHE_0_REGS)->L1PCC, CACHE_L1PCC_POPER);
    /* End Critical Section */
    _restore_interrupts(restore_value);
    
    if(oldL1PFrz == CSL_CACHE_L1PCC_OPER_FREEZE)
        return CACHE_L1P_FREEZE;
    else
        return CACHE_L1P_NORMAL;
}

/* ============================================================================
 *    CACHE_unfreezeL1p
 *  ============================================================================
 */
#pragma CODE_SECTION (CACHE_unfreezeL1p, ".text:csl_section:cache");
CACHE_L1_Freeze CACHE_unfreezeL1p (void)
{
    Uint32      temp;
    Uint32      restore_value;
     
    /* Critical section */
    restore_value = _disable_interrupts();
    
    ((CSL_CacheRegsOvly)CSL_CACHE_0_REGS)->L1PCC = 
                          CSL_FMK(CACHE_L1PCC_OPER, CSL_CACHE_L1PCC_OPER_NORM);
    temp = CSL_FEXT(((CSL_CacheRegsOvly) \
                                  CSL_CACHE_0_REGS)->L1PCC, CACHE_L1PCC_POPER);
    
    /* End Critical Section */
    _restore_interrupts(restore_value);
        
    if(temp == CSL_CACHE_L1PCC_OPER_NORM)
        return CACHE_L1P_NORMAL;
    else
        return CACHE_L1P_FREEZE;
}

/* ============================================================================
 *    CACHE_invL1p
 *  ============================================================================
 */
#pragma CODE_SECTION (CACHE_invL1p, ".text:csl_section:cache");
void CACHE_invL1p (
    void            *blockPtr,
    Uint32          byteCnt,
    CACHE_Wait      wait
)
{
    Uint32      restore_value;
     
    CACHE_waitInternal();
    
    /* Critical section */
    restore_value = _disable_interrupts();
    
    ((CSL_CacheRegsOvly)CSL_CACHE_0_REGS)->L1PIBAR = (Uint32)blockPtr;
    ((CSL_CacheRegsOvly)CSL_CACHE_0_REGS)->L1PIWC = ((byteCnt+3)>>2);
    _CSL_cachebusyState = CACHE_WAIT_L1PINV; 
    
    /* End Critical section */
    _restore_interrupts(restore_value);
        
    if (wait)
        _CACHE_wait(wait);
}

/* ============================================================================
 *    CACHE_invAllL1p
 *  ============================================================================
 */
#pragma CODE_SECTION (CACHE_invAllL1p, ".text:csl_section:cache");
void CACHE_invAllL1p (
    CACHE_Wait      wait
)
{
    Uint32      restore_value;

    CACHE_waitInternal();
     
    /* Critical Section */
    restore_value = _disable_interrupts();
    
    ((CSL_CacheRegsOvly)CSL_CACHE_0_REGS)->L1PINV = 1;
    _CSL_cachebusyState = CACHE_WAIT_L1PINVALL;
    
    /* End Critical Section */
    _restore_interrupts(restore_value);
        
    if (wait)
        _CACHE_wait(wait);
}

/* ============================================================================
 *    CACHE_setL1dSize
 *  ============================================================================
 */
#pragma CODE_SECTION (CACHE_setL1dSize, ".text:csl_section:cache");
CACHE_L1Size CACHE_setL1dSize (
    CACHE_L1Size    newSize
)
{
    Uint32      curSize;
    Uint32      restore_value;
     
    curSize = ((CSL_CacheRegsOvly)CSL_CACHE_0_REGS)->L1DCFG;
    
    /* Critical Section */
    restore_value = _disable_interrupts();
    
    ((CSL_CacheRegsOvly)CSL_CACHE_0_REGS)->L1DCFG = newSize;
    newSize = (CACHE_L1Size)((CSL_CacheRegsOvly)CSL_CACHE_0_REGS)->L1DCFG;
    
    /* End critical section */
    _restore_interrupts(restore_value);
    
    return (CACHE_L1Size) (curSize);
}

/* ============================================================================
 *    CACHE_freezeL1d
 *  ============================================================================
 */
#pragma CODE_SECTION (CACHE_freezeL1d, ".text:csl_section:cache");
CACHE_L1_Freeze CACHE_freezeL1d (void)
{

    Uint32 temp;
    Uint32 restore_value;
     
    /* critical section */
    restore_value = _disable_interrupts();
    
    ((CSL_CacheRegsOvly)CSL_CACHE_0_REGS)->L1DCC = 
                        CSL_FMK(CACHE_L1DCC_OPER, CSL_CACHE_L1DCC_OPER_FREEZE);
    temp = CSL_FEXT(((CSL_CacheRegsOvly) \
                                  CSL_CACHE_0_REGS)->L1DCC, CACHE_L1DCC_POPER);
    
    /* End critical section */
    _restore_interrupts(restore_value);
        
    if(temp == CSL_CACHE_L1DCC_OPER_FREEZE)
        return CACHE_L1D_FREEZE;
    else
        return CACHE_L1D_NORMAL;
}

/* ============================================================================
 *    CACHE_unfreezeL1d
 *  ============================================================================
 */
#pragma CODE_SECTION (CACHE_unfreezeL1d, ".text:csl_section:cache");
CACHE_L1_Freeze CACHE_unfreezeL1d (void)
{
    Uint32      temp;
    Uint32      restore_value;
    
    /* critical section */
    restore_value = _disable_interrupts();
     
    
    ((CSL_CacheRegsOvly)CSL_CACHE_0_REGS)->L1DCC = 
                        CSL_FMK(CACHE_L1DCC_OPER, CSL_CACHE_L1DCC_OPER_NORM);
    temp = CSL_FEXT(((CSL_CacheRegsOvly) \
                                CSL_CACHE_0_REGS)->L1DCC, CACHE_L1DCC_POPER);
    
    /* End critical section */
    _restore_interrupts(restore_value);
    
    if(temp == CSL_CACHE_L1DCC_OPER_NORM)
        return CACHE_L1D_NORMAL;
    else
        return CACHE_L1D_FREEZE;
}

/* ============================================================================
 *    CACHE_wbL1d
 *  ============================================================================
 */
#pragma CODE_SECTION (CACHE_wbL1d, ".text:csl_section:cache");
void CACHE_wbL1d (
    void            *blockPtr,
    Uint32          byteCnt,
    CACHE_Wait      wait
)
{
    Uint32  restore_value;

    CACHE_waitInternal();
    
    /* Critical section */
    restore_value = _disable_interrupts();

    
    ((CSL_CacheRegsOvly)CSL_CACHE_0_REGS)->L1DWBAR = (Uint32)blockPtr;
    ((CSL_CacheRegsOvly)CSL_CACHE_0_REGS)->L1DWWC =((byteCnt+3)>>2);
    _CSL_cachebusyState = CACHE_WAIT_L1DWB;
    _CSL_cacheEmifState = (CACHE_emifState) \
                        (CSL_CACHE_EMIF_ISRANGE((Uint32)blockPtr) 
                             + CSL_CACHE_EMIF_ISEMIFBRANGE((Uint32)blockPtr));
    
    /* End critical section  */
    _restore_interrupts(restore_value);
    
    if (wait)
        _CACHE_wait(wait);
}

/* ============================================================================
 *    CACHE_invL1d
 *  ============================================================================
 */
#pragma CODE_SECTION (CACHE_invL1d, ".text:csl_section:cache");
void CACHE_invL1d (
    void         *blockPtr,
    Uint32       byteCnt,
    CACHE_Wait   wait
)
{
    Uint32 restore_value;
     
    CACHE_waitInternal();
    
    /* critical section */
    restore_value = _disable_interrupts();
    
    ((CSL_CacheRegsOvly)CSL_CACHE_0_REGS)->L1DIBAR = (Uint32)blockPtr;
    ((CSL_CacheRegsOvly)CSL_CACHE_0_REGS)->L1DIWC = ((byteCnt+3)>>2);
    _CSL_cachebusyState = CACHE_WAIT_L1DINV;
    
    /* End critical section */
    _restore_interrupts(restore_value);
    
    if (wait)
        _CACHE_wait(wait);
}

/* ============================================================================
 *    CACHE_wbInvL1d
 *  ============================================================================
 */
#pragma CODE_SECTION (CACHE_wbInvL1d, ".text:csl_section:cache");
void CACHE_wbInvL1d (
    void         *blockPtr,
    Uint32       byteCnt,
    CACHE_Wait   wait
)
{
    Uint32 restore_value;

    CACHE_waitInternal();
    
    /* critical section */
    restore_value = _disable_interrupts();
    
    ((CSL_CacheRegsOvly)CSL_CACHE_0_REGS)->L1DWIBAR = (Uint32)blockPtr;
    ((CSL_CacheRegsOvly)CSL_CACHE_0_REGS)->L1DWIWC = ((byteCnt+3)>>2);
    _CSL_cachebusyState = CACHE_WAIT_L1DWBINV;
    _CSL_cacheEmifState = (CACHE_emifState) \
                        (CSL_CACHE_EMIF_ISRANGE((Uint32)blockPtr) \
                            + CSL_CACHE_EMIF_ISEMIFBRANGE((Uint32)blockPtr));
                            
    /* End critical section */
    _restore_interrupts(restore_value);
        
    if (wait)
        _CACHE_wait(wait);
}

/* ============================================================================
 *    CACHE_wbAllL1d
 *  ============================================================================
 */
#pragma CODE_SECTION (CACHE_wbAllL1d, ".text:csl_section:cache");
void CACHE_wbAllL1d (
    CACHE_Wait     wait
)
{
    Uint32 restore_value;

    CACHE_waitInternal();
    
    /* critical section */
    restore_value = _disable_interrupts();
    
    ((CSL_CacheRegsOvly)CSL_CACHE_0_REGS)->L1DWB = 1;
    _CSL_cachebusyState = CACHE_WAIT_L1DWBALL;  
    _CSL_cacheEmifState = CACHE_EMIF_AB;
    
    /*  End critical section */
    _restore_interrupts(restore_value);
    
    if (wait)
        _CACHE_wait(wait);
}

/* ============================================================================
 *    CACHE_invAllL1d
 *  ============================================================================
 */
#pragma CODE_SECTION (CACHE_invAllL1d, ".text:csl_section:cache");
void CACHE_invAllL1d (
    CACHE_Wait    wait
)
{
    Uint32 restore_value;

    CACHE_waitInternal();
    
    /* critical section */
    restore_value = _disable_interrupts();
      
    ((CSL_CacheRegsOvly)CSL_CACHE_0_REGS)->L1DINV = 1;
    _CSL_cachebusyState = CACHE_WAIT_L1DINVALL;
    
    /* End critical section */
    _restore_interrupts(restore_value);
    
    if (wait)
        _CACHE_wait(wait);
}

/* ============================================================================
 *    CACHE_wbInvAllL1d
 *  ============================================================================
 */
#pragma CODE_SECTION (CACHE_wbInvAllL1d, ".text:csl_section:cache");
void CACHE_wbInvAllL1d (
    CACHE_Wait      wait
)
{
    Uint32 restore_value;
     
    CACHE_waitInternal();
    
    /* critical section */
    restore_value = _disable_interrupts();
    
    ((CSL_CacheRegsOvly)CSL_CACHE_0_REGS)->L1DWBINV = 1;
    _CSL_cachebusyState = CACHE_WAIT_L1DWBINVALL;
    _CSL_cacheEmifState = CACHE_EMIF_AB;
    
    /* End of critical section */
    _restore_interrupts(restore_value);
    
    if (wait)
        _CACHE_wait(wait);
}
