/*  ============================================================================
 * Copyright (c) Texas Instruments Inc 2002, 2003, 2004, 2005, 2006, 2007, 2008
 *
 *   Use of this software is controlled by the terms and conditions found in the
 *   license agreement under which this software has been supplied.
 *  ============================================================================
 */

/** ============================================================================
 *   @file  csl_cacheL2.c
 *
 *   @path  $(CSLPATH)\src\cache
 *
 *   @desc  File for functional layer of L2 Cache
 *
 */
 
/* =============================================================================
 *  Revision History
 *  ===============
 *  23-Mar-2004 Jamon Bowen File Created
 *
 *  21-Jun-2004 Ruchika Kharwar modified
 *
 *  24-Jan-2006 ds Updated CACHE_setL2Size() API to return Old value  
 *  08-Sep-2008 Update for Doxygen
 * ============================================================================= 
 */

#include <csl_cache.h>
#include <_csl_cache.h>

/* ============================================================================
 *    CACHE_setL2Size
 *  ============================================================================
 */
#pragma CODE_SECTION (CACHE_setL2Size, ".text:csl_section:cache");
CACHE_L2Size CACHE_setL2Size (
    CACHE_L2Size    newSize
)
{
    Uint32 restore_value;
    Uint32         curL2cfg;
    CACHE_L2Size   returnVal;
     
    curL2cfg = ((CSL_CacheRegsOvly)CSL_CACHE_0_REGS)->L2CFG;
    returnVal = (CACHE_L2Size) CSL_FEXT(curL2cfg, CACHE_L2CFG_MODE);
    
    /* critical section */
    restore_value = _disable_interrupts();
    
    CSL_FINS(((CSL_CacheRegsOvly)CSL_CACHE_0_REGS)->L2CFG, CACHE_L2CFG_MODE, newSize);
    newSize = (CACHE_L2Size)((CSL_CacheRegsOvly)CSL_CACHE_0_REGS)->L2CFG;
    
    /* End critical section */
    _restore_interrupts(restore_value);
    
    return returnVal;
}

/* ============================================================================
 *    CACHE_setL2Mode
 *  ============================================================================
 */
#pragma CODE_SECTION (CACHE_setL2Mode, ".text:csl_section:cache");
CACHE_L2Mode CACHE_setL2Mode(
    CACHE_L2Mode    newMode
)
{
    Uint32          oldval;
    CACHE_L2Mode    returnVal;
    Uint32          restore_value; 

    /* critical section */
    restore_value = _disable_interrupts();
    
    oldval = ((CSL_CacheRegsOvly)CSL_CACHE_0_REGS)->L2CFG;
    returnVal = (CACHE_L2Mode)(CSL_FEXT(oldval,CACHE_L2CFG_L2CC));
    ((CSL_CacheRegsOvly)CSL_CACHE_0_REGS)->L2CFG = 
                                   CSL_FINS(oldval,CACHE_L2CFG_L2CC, newMode);
    oldval = ((CSL_CacheRegsOvly)CSL_CACHE_0_REGS)->L2CFG;
    
    /* End critical section */
    _restore_interrupts(restore_value);
    
    return returnVal;
}

/* ============================================================================
 *    CACHE_wbL2
 *  ============================================================================
 */
#pragma CODE_SECTION (CACHE_wbL2, ".text:csl_section:cache");
void CACHE_wbL2 (
    void         *blockPtr,
    Uint32       byteCnt,
    CACHE_Wait   wait
)
{
    Uint32  restore_value;
     
    CACHE_waitInternal();

    /* critical section */
    restore_value = _disable_interrupts();
    
    ((CSL_CacheRegsOvly)CSL_CACHE_0_REGS)->L2WBAR = (Uint32)blockPtr;
    ((CSL_CacheRegsOvly)CSL_CACHE_0_REGS)->L2WWC =((byteCnt+3)>>2);  
    _CSL_cachebusyState = CACHE_WAIT_L2WB;
    _CSL_cacheEmifState = 
        (CACHE_emifState)(CSL_CACHE_EMIF_ISRANGE((Uint32)blockPtr) \
                            + CSL_CACHE_EMIF_ISEMIFBRANGE((Uint32)blockPtr));
    
    /* End critical section */
    _restore_interrupts(restore_value);
        
    if(wait)
        _CACHE_wait(wait);
}

/* ============================================================================
 *    CACHE_invL2
 *  ============================================================================
 */
#pragma CODE_SECTION (CACHE_invL2, ".text:csl_section:cache");
void CACHE_invL2 (
    void          *blockPtr,
    Uint32        byteCnt,
    CACHE_Wait    wait
)
{
    Uint32  restore_value;

    CACHE_waitInternal();
    
    /* critical section */
    restore_value = _disable_interrupts();
    
    ((CSL_CacheRegsOvly)CSL_CACHE_0_REGS)->L2IBAR = (Uint32)blockPtr;
    ((CSL_CacheRegsOvly)CSL_CACHE_0_REGS)->L2IWC = ((byteCnt+3)>>2);       
    _CSL_cachebusyState = CACHE_WAIT_L2INV; 
    
    /* End critical section */
    _restore_interrupts(restore_value);
        
    if(wait)
        _CACHE_wait(wait);
}

/* ============================================================================
 *    CACHE_wbInvL2
 *  ============================================================================
 */
#pragma CODE_SECTION (CACHE_wbInvL2, ".text:csl_section:cache");
void CACHE_wbInvL2 (
    void         *blockPtr,
    Uint32       byteCnt,
    CACHE_Wait   wait
)
{
    Uint32  restore_value;
     
    CACHE_waitInternal();
       
    /* critical section */
    restore_value = _disable_interrupts();
    
    ((CSL_CacheRegsOvly)CSL_CACHE_0_REGS)->L2WIBAR = (Uint32)blockPtr;
    ((CSL_CacheRegsOvly)CSL_CACHE_0_REGS)->L2WIWC = ((byteCnt+3)>>2);
    _CSL_cachebusyState = CACHE_WAIT_L2WBINV;
    _CSL_cacheEmifState = 
        (CACHE_emifState)(CSL_CACHE_EMIF_ISRANGE((Uint32)blockPtr) \
                            + CSL_CACHE_EMIF_ISEMIFBRANGE((Uint32)blockPtr));
    
    /* End critical section */
	_restore_interrupts(restore_value);
    
    if(wait)
        _CACHE_wait(wait);
}

/* ============================================================================
 *    CACHE_wbAllL2
 *  ============================================================================
 */
#pragma CODE_SECTION (CACHE_wbAllL2, ".text:csl_section:cache");
void CACHE_wbAllL2 (
    CACHE_Wait      wait
)
{
    Uint32  restore_value;

    CACHE_waitInternal();
    
    /* critical section */
    restore_value = _disable_interrupts();
    
    ((CSL_CacheRegsOvly)CSL_CACHE_0_REGS)->L2WB = 1; 
    _CSL_cachebusyState = CACHE_WAIT_L2WBALL;
    _CSL_cacheEmifState = CACHE_EMIF_AB;
    
    /* End critical section */
    _restore_interrupts(restore_value);
    
    if(wait)
        _CACHE_wait(wait);
}

/* ============================================================================
 *    CACHE_invAllL2
 *  ============================================================================
 */
#pragma CODE_SECTION (CACHE_invAllL2, ".text:csl_section:cache");
void CACHE_invAllL2 (
    CACHE_Wait      wait
)
{
    Uint32  restore_value;

    CACHE_waitInternal(); 
      
    /* critical section */
    restore_value = _disable_interrupts();
    
    ((CSL_CacheRegsOvly)CSL_CACHE_0_REGS)->L2INV = 1; 
    _CSL_cachebusyState = CACHE_WAIT_L2INVALL;
    
    _restore_interrupts(restore_value);
    /* End critical section */
    
    if(wait)
        _CACHE_wait(wait);
}

/* ============================================================================
 *    CACHE_wbInvAllL2
 *  ============================================================================
 */
#pragma CODE_SECTION (CACHE_wbInvAllL2, ".text:csl_section:cache");
void CACHE_wbInvAllL2 (
    CACHE_Wait      wait
)
{
    Uint32  restore_value;
     
    CACHE_waitInternal();
    
    /* critical section */
    restore_value = _disable_interrupts();
    
    ((CSL_CacheRegsOvly)CSL_CACHE_0_REGS)->L2WBINV = 1; 
    _CSL_cachebusyState = CACHE_WAIT_L2WBINVALL;
    _CSL_cacheEmifState = CACHE_EMIF_AB;
    
    /* End critical section */
    _restore_interrupts(restore_value);
    
    if(wait)
        _CACHE_wait(wait);
}

