/**  ===========================================================================
 *   Copyright (c) Texas Instruments Inc 2002, 2003, 2004, 2005, 2006, 2007, 2008
 *
 *   Use of this software is controlled by the terms and conditions found in the
 *   license agreement under which this software has been supplied.
 *   ===========================================================================
 */

/** ============================================================================
 * @file  csl_dat.c
 *
 * @desc  This file defines the functions in the DAT module.
 * 
 *  @path  $(CSLPATH)\src\dat
 */
  
/* =============================================================================
 *  Revision History
 *  ===============
 *  14-Nov-2005 ds File modified - In the APIs DAT_copy(), DAT_copy2d(), 
 *                                 DAT_fill() added macro to set the STATIC when
 *                                 OPT is programmed.   
 *  08-Sep-2008 Update for Doxygen
 * =============================================================================
 */
 
#include <csl_dat.h>
#include <_csl_dat.h>

/* ============================================================================
 *    DAT_open 
 * ===========================================================================
 */
#pragma CODE_SECTION (DAT_open, ".text:csl_section:dat");
Int16 DAT_open (
    DAT_Setup   *setup
)
{
    Uint32    cs;
    
    /* Enter critical section */
    cs = _disable_interrupts ();

    _CSL_datStateStruct.qchNum = setup->qchNum;
    _CSL_datStateStruct.regionNum = setup->regionNum;
    _CSL_datStateStruct.tccNum = setup->tccNum;
    _CSL_datStateStruct.priority = setup->priority;
    _CSL_datStateStruct.paramNum = setup->paramNum;
    _CSL_datStateStruct.pending = FALSE;

    /* the que entry number mapping */
    CSL_FINSR (((CSL_Edma3ccRegsOvly) CSL_EDMA3CC_0_REGS)->QDMAQNUM,
               setup->qchNum * 4 + 2, setup->qchNum * 4, setup->priority);

    /* The parameter set mapping */
    ((CSL_Edma3ccRegsOvly) CSL_EDMA3CC_0_REGS)->QCHMAP[setup->qchNum] =
        CSL_FMK (EDMA3CC_QCHMAP_PAENTRY, setup->paramNum) |
        CSL_FMK (EDMA3CC_QCHMAP_TRWORD, 7);

    /* Enable the DRAE Bit */
    if (setup->regionNum != CSL_DAT_REGION_GLOBAL) {
        ((CSL_Edma3ccRegsOvly) CSL_EDMA3CC_0_REGS)->QRAE[setup->regionNum] |=
            (1 << setup->qchNum);
        ((CSL_Edma3ccRegsOvly)
            CSL_EDMA3CC_0_REGS)->SHADOW[setup->regionNum].QEESR =
            (1 << setup->qchNum);
    }
    else {
        ((CSL_Edma3ccRegsOvly) CSL_EDMA3CC_0_REGS)->QEESR =
            (1 << setup->qchNum);
    }
    
    /* Leave critical section */
    _restore_interrupts (cs);
    
    return CSL_SOK;
}

/* ============================================================================
 *    DAT_close 
 * ===========================================================================
 */
#pragma CODE_SECTION (DAT_close, ".text:csl_section:dat");
void DAT_close (void)
{
    Uint32    cs;

    if (_CSL_datStateStruct.regionNum != CSL_DAT_REGION_GLOBAL) {
        
        cs = _disable_interrupts ();
        
        ((CSL_Edma3ccRegsOvly)
            CSL_EDMA3CC_0_REGS)->QRAE[_CSL_datStateStruct.regionNum] &=
            ~(1 << _CSL_datStateStruct.qchNum);
            
        _restore_interrupts (cs);
    }
}

/* ============================================================================
 *    DAT_copy 
 * ===========================================================================
 */
#pragma CODE_SECTION (DAT_copy, ".text:csl_section:dat");
Uint32 DAT_copy (
    void      *src, 
    void      *dst,
    Uint16    byteCnt
)
{
    volatile    CSL_Edma3ccParamsetRegs *param;
    Uint32      cs;

    cs = _disable_interrupts ();

    param = &((CSL_Edma3ccRegsOvly)
        CSL_EDMA3CC_0_REGS)->PARAMSET[_CSL_datStateStruct.paramNum];
    if (_CSL_datStateStruct.pending)
        DAT_wait (_CSL_datStateStruct.tccNum);
        
    /* The options in the param set programmed need to have STATIC = 1.
     * If STATIC != 1 then when QDMA links to NULL SER will get set causing
     * further issue of Transfer requests
     */
    param->OPT = CSL_FMKT (EDMA3CC_OPT_TCINTEN, ENABLE) |
        CSL_FMK (EDMA3CC_OPT_TCC, _CSL_datStateStruct.tccNum) |
        CSL_FMKT (EDMA3CC_OPT_STATIC, STATIC);
        
    param->SRC = (Uint32) (src);
    param->A_B_CNT = CSL_FMK (EDMA3CC_A_B_CNT_ACNT, byteCnt) |
        CSL_FMK (EDMA3CC_A_B_CNT_BCNT, 1);
    param->DST = (Uint32) (dst);
    param->LINK_BCNTRLD = CSL_FMK (EDMA3CC_LINK_BCNTRLD_LINK, 0xFFFF);
    param->CCNT = 0x00000001;
    _CSL_datStateStruct.pending = TRUE;
    _restore_interrupts (cs);
    return _CSL_datStateStruct.tccNum;
}

/* ============================================================================
 *    DAT_fill 
 * ===========================================================================
 */
#pragma CODE_SECTION (DAT_fill, ".text:csl_section:dat");
Uint32 DAT_fill (
    void      *dst,
    Uint16    byteCnt, 
    Uint32    *value
)
{
    volatile CSL_Edma3ccParamsetRegs *param;
    Uint32 cs;

    cs = _disable_interrupts ();

    param = &((CSL_Edma3ccRegsOvly)
        CSL_EDMA3CC_0_REGS)->PARAMSET[_CSL_datStateStruct.paramNum];

    if (_CSL_datStateStruct.pending)
        DAT_wait (_CSL_datStateStruct.tccNum);
        
    /* The options in the param set programmed need to have STATIC = 1.
     * If STATIC != 1 then when QDMA links to NULL SER will get set causing
     * further issue of Transfer requests
     */
    param->OPT = CSL_FMKT (EDMA3CC_OPT_TCINTEN, ENABLE) |
        CSL_FMK (EDMA3CC_OPT_TCC, _CSL_datStateStruct.tccNum) |
        CSL_FMKT (EDMA3CC_OPT_SYNCDIM, ABSYNC) |
        CSL_FMKT (EDMA3CC_OPT_STATIC, STATIC);
        
    param->SRC = (Uint32) (value);
    param->A_B_CNT = CSL_FMK (EDMA3CC_A_B_CNT_ACNT, 1) |
        CSL_FMK (EDMA3CC_A_B_CNT_BCNT, byteCnt);
    param->DST = (Uint32) (dst);
    param->SRC_DST_BIDX = CSL_FMK (EDMA3CC_SRC_DST_BIDX_DSTBIDX, 1) |
        CSL_FMK (EDMA3CC_SRC_DST_BIDX_SRCBIDX, 0);
    param->LINK_BCNTRLD = CSL_FMK (EDMA3CC_LINK_BCNTRLD_LINK, 0xFFFF);
    param->CCNT = 0x00000001;
    _CSL_datStateStruct.pending = TRUE;
    _restore_interrupts (cs);
    return _CSL_datStateStruct.tccNum;
}

/* ============================================================================
 *    DAT_wait 
 * ===========================================================================
 */
#pragma CODE_SECTION (DAT_wait, ".text:csl_section:dat");
void DAT_wait ( 
    Uint32    id
)
{
    Uint32 cs;

    cs = _disable_interrupts ();

    if (_CSL_datStateStruct.regionNum != CSL_DAT_REGION_GLOBAL) {
        if (_CSL_datStateStruct.tccNum < 32) {
            while (!(((CSL_Edma3ccRegsOvly)
                        CSL_EDMA3CC_0_REGS)->SHADOW[_CSL_datStateStruct.
                        regionNum].IPR & (1 << _CSL_datStateStruct.tccNum)));
            ((CSL_Edma3ccRegsOvly)
                CSL_EDMA3CC_0_REGS)->SHADOW[_CSL_datStateStruct.regionNum].ICR =
                (1 << _CSL_datStateStruct.tccNum);
        }
        else {
            while (!(((CSL_Edma3ccRegsOvly)
                        CSL_EDMA3CC_0_REGS)->SHADOW[_CSL_datStateStruct.
                        regionNum].IPRH & (1 << (_CSL_datStateStruct.tccNum -
                            32))));
            ((CSL_Edma3ccRegsOvly)
                CSL_EDMA3CC_0_REGS)->SHADOW[_CSL_datStateStruct.regionNum].
                ICRH = (1 << _CSL_datStateStruct.tccNum - 32);
        }
    }
    else {
        if (_CSL_datStateStruct.tccNum < 32) {
            while (!(((CSL_Edma3ccRegsOvly) CSL_EDMA3CC_0_REGS)->IPR &
                    (1 << _CSL_datStateStruct.tccNum)));
            ((CSL_Edma3ccRegsOvly) CSL_EDMA3CC_0_REGS)->ICR =
                (1 << _CSL_datStateStruct.tccNum);
        }
        else {
            while (!(((CSL_Edma3ccRegsOvly) CSL_EDMA3CC_0_REGS)->IPRH &
                    (1 << (_CSL_datStateStruct.tccNum - 32))));
            ((CSL_Edma3ccRegsOvly) CSL_EDMA3CC_0_REGS)->ICRH =
                (1 << _CSL_datStateStruct.tccNum - 32);
        }
    }
    _CSL_datStateStruct.pending = FALSE;
    _restore_interrupts (cs);
}

/* ============================================================================
 *    DAT_busy 
 *  ============================================================================
 */
#pragma CODE_SECTION (DAT_busy, ".text:csl_section:dat");
Int16 DAT_busy (
    Uint32    id
)
{
    Bool        status = TRUE;
    Uint32      temp;
    Uint32      cs;

    cs = _disable_interrupts ();
    
     /* The number 32 signifies lower 32 channels */
    if (_CSL_datStateStruct.regionNum != CSL_DAT_REGION_GLOBAL) {
        if (_CSL_datStateStruct.tccNum < 32) {
            temp = ((CSL_Edma3ccRegsOvly)
                CSL_EDMA3CC_0_REGS)->SHADOW[_CSL_datStateStruct.regionNum].IPR &
                (1 << _CSL_datStateStruct.tccNum);
        }
        else {
            temp = ((CSL_Edma3ccRegsOvly)
                CSL_EDMA3CC_0_REGS)->SHADOW[_CSL_datStateStruct.regionNum].
                IPRH & (1 << (_CSL_datStateStruct.tccNum - 32));
        }
    }
    else {
        if (_CSL_datStateStruct.tccNum < 32) {
            temp = ((CSL_Edma3ccRegsOvly) CSL_EDMA3CC_0_REGS)->IPR &
                (1 << _CSL_datStateStruct.tccNum);
        }
        else {
            temp = ((CSL_Edma3ccRegsOvly) CSL_EDMA3CC_0_REGS)->IPRH &
                (1 << (_CSL_datStateStruct.tccNum - 32));
        }
    }
    
    if (temp) {
        if (_CSL_datStateStruct.regionNum != CSL_DAT_REGION_GLOBAL) {
            if (_CSL_datStateStruct.tccNum < 32) {
                ((CSL_Edma3ccRegsOvly)
                    CSL_EDMA3CC_0_REGS)->SHADOW[_CSL_datStateStruct.regionNum].
                    ICR = (1 << _CSL_datStateStruct.tccNum);
            }
            else {
                ((CSL_Edma3ccRegsOvly)
                    CSL_EDMA3CC_0_REGS)->SHADOW[_CSL_datStateStruct.regionNum].
                    ICRH = (1 << (_CSL_datStateStruct.tccNum - 32));
            }
        }
        else {
            if (_CSL_datStateStruct.tccNum < 32) {
                ((CSL_Edma3ccRegsOvly) CSL_EDMA3CC_0_REGS)->ICR =
                    (1 << _CSL_datStateStruct.tccNum);
            }
            else {
                ((CSL_Edma3ccRegsOvly) CSL_EDMA3CC_0_REGS)->ICRH =
                    (1 << (_CSL_datStateStruct.tccNum - 32));
            }
        }
        
        _CSL_datStateStruct.pending = FALSE;
        status = FALSE;
    }
    
    _restore_interrupts (cs);
    
    return status;
}

/* ============================================================================
 *    DAT_copy2d 
 * ===========================================================================
 */
#pragma CODE_SECTION (DAT_copy2d, ".text:csl_section:dat");
Uint32 DAT_copy2d (
    Uint32    type, 
    void      *src, 
    void      *dst, 
    Uint16    lineLen,
    Uint16    lineCnt,
    Uint16    linePitch
)
{
    volatile CSL_Edma3ccParamsetRegs *param;
    Uint32 cs;

    cs = _disable_interrupts ();

    param = &((CSL_Edma3ccRegsOvly)
        CSL_EDMA3CC_0_REGS)->PARAMSET[_CSL_datStateStruct.paramNum];
        
    if (_CSL_datStateStruct.pending) 
        DAT_wait (_CSL_datStateStruct.tccNum);
    
    /* The options in the param set programmed need to have STATIC = 1.
     * If STATIC != 1 then when QDMA links to NULL SER will get set causing
     * further issue of Transfer requests
     */    
    param->OPT = CSL_FMKT (EDMA3CC_OPT_TCINTEN, ENABLE) |
        CSL_FMK (EDMA3CC_OPT_TCC, _CSL_datStateStruct.tccNum) |
        CSL_FMKT (EDMA3CC_OPT_SYNCDIM, ABSYNC) |
        CSL_FMKT (EDMA3CC_OPT_STATIC, STATIC);

    param->SRC = (Uint32) (src);
    
    param->A_B_CNT = CSL_FMK (EDMA3CC_A_B_CNT_ACNT, lineLen) |
        CSL_FMK (EDMA3CC_A_B_CNT_BCNT, lineCnt);
        
    param->DST = (Uint32) (dst);
    
    if (type == DAT_2D2D) {
        param->SRC_DST_BIDX =
            CSL_FMK (EDMA3CC_SRC_DST_BIDX_DSTBIDX,
            linePitch) | CSL_FMK (EDMA3CC_SRC_DST_BIDX_SRCBIDX, linePitch);
    }
    else {
        if (type == DAT_1D2D) {
            param->SRC_DST_BIDX =
                CSL_FMK (EDMA3CC_SRC_DST_BIDX_DSTBIDX,
                linePitch) | CSL_FMK (EDMA3CC_SRC_DST_BIDX_SRCBIDX, lineLen);
        }
        else {
            if (type == DAT_2D1D) {
                param->SRC_DST_BIDX = 
                    CSL_FMK (EDMA3CC_SRC_DST_BIDX_DSTBIDX, lineLen) |
                    CSL_FMK (EDMA3CC_SRC_DST_BIDX_SRCBIDX, linePitch);
            }
        }
    }

    param->LINK_BCNTRLD = CSL_FMK (EDMA3CC_LINK_BCNTRLD_LINK, 0xFFFF);
    param->CCNT = 0x00000001;
    
    _CSL_datStateStruct.pending = TRUE;
    
    _restore_interrupts (cs);
    
    return _CSL_datStateStruct.tccNum;
}

/* ============================================================================
 *    DAT_setPriority 
 *  ============================================================================
 */
#pragma CODE_SECTION (DAT_setPriority, ".text:csl_section:dat");
void DAT_setPriority (
    Int    priority
)
{
    Uint32 cs;

    if (_CSL_datStateStruct.pending)
        DAT_wait (_CSL_datStateStruct.tccNum);
        
    cs = _disable_interrupts ();
    
    /* The que entry number mapping */
    /* The numbers 4 and 2 used to calculate the MSB and LSB of qdma Qnumber */
    CSL_FINSR (((CSL_Edma3ccRegsOvly) CSL_EDMA3CC_0_REGS)->QDMAQNUM,
        _CSL_datStateStruct.qchNum * 4 + 2,
        _CSL_datStateStruct.qchNum * 4, priority);
    
    _CSL_datStateStruct.priority = priority;
    
    _restore_interrupts (cs);
}
