/*  ============================================================================
 *   Copyright (c) Texas Instruments Inc 2002, 2003, 2004, 2005, 2008
 *
 *   Use of this software is controlled by the terms and conditions found
 *   in the license agreement under which this software has been supplied.
 *  ============================================================================
 */

/** 
 *  @file    csl_ddr2GetHwSetup.c
 *
 *  @brief   File for functional layer of CSL API @a CSL_ddr2GetHwSetup()
 *
 *  @path    $(CSLPATH)\src\ddr2
 *
 *  @desc
 *    - The @a CSL_ddr2GetHwSetup() function definition & it's associated
 *      functions
 *
 *  Modification 1
 *    - Modified on: 12/4/2005
 *    - Reason: created the sources
 *
 *  @date 12th April, 2005
 *  @author Ramitha Mathew.
 *
 */
/*  ============================================================================
 *  Revision History
 *  ================
 *  30-May-2008 Updated SDCFG register bit field names according to UG.
 *  08-Sep-2008 Update for Doxygen
 *  11-Nov-2008 Added fields ddr2Ddqs, ddr2Term0, ddr2Term1, ddrDisDll, sdramDrive0. 
 *              Updated DDR2_SDTIM2_T_XSNR and DDR2_SDTIM2_T_XSRD field according to UG
 *  ============================================================================
 */

#include <csl_ddr2.h>


#pragma CODE_SECTION (CSL_ddr2GetHwSetup, ".text:csl_section:ddr2")
CSL_Status  CSL_ddr2GetHwSetup(
    CSL_Ddr2Handle     hDdr2,
    CSL_Ddr2HwSetup    *setup
)
{
    CSL_Status status = CSL_SOK;
	Uint32 tmpReg;
    
    /* invalid parameter checking */
    if (setup == NULL) {
        return CSL_ESYS_INVPARAMS;
    }
    
    /* bad handle checking */
    if (hDdr2 == NULL) {
        return CSL_ESYS_BADHANDLE;
    }    

    /* Get the SDRAM Refresh Rate Configuration */
    setup->refreshRate = (Uint16)CSL_FEXT(hDdr2->regs->SDRFC, 
                                 DDR2_SDRFC_REFRESH_RATE);
    setup->readLatncy = (Uint8)CSL_FEXT(hDdr2->regs->DMCCTL, DDR2_DMCCTL_RL);
     

    /* Get the SDRAM Configuration */
    if (setup->setParam != NULL) {
	    tmpReg = hDdr2->regs->SDCFG;
        setup->setParam->casLatncy = 
                           (CSL_Ddr2CasLatency)CSL_FEXT(tmpReg, DDR2_SDCFG_CL);
        setup->setParam->ibank = 
                           (CSL_Ddr2IntBank)CSL_FEXT(tmpReg, DDR2_SDCFG_IBANK);
        setup->setParam->pageSize = 
                       (CSL_Ddr2PageSize)CSL_FEXT(tmpReg, DDR2_SDCFG_PAGESIZE);
        setup->setParam->narrowMode = 
                                 (CSL_Ddr2Mode)CSL_FEXT(tmpReg, DDR2_SDCFG_NM);
		setup->setParam->ddr2En = CSL_FEXT(tmpReg, DDR2_SDCFG_DDR2_ENABLE);
		setup->setParam->ddrEn = CSL_FEXT(tmpReg, DDR2_SDCFG_DDR_ENABLE);
		setup->setParam->sdramEn = CSL_FEXT(tmpReg, DDR2_SDCFG_SDRAM_ENABLE);
		setup->setParam->ddr2Ddqs = (CSL_Ddr2Differential)CSL_FEXT(tmpReg, DDR2_SDCFG_DDR2_DDQS);
		setup->setParam->ddr2Term0 = CSL_FEXT(tmpReg, DDR2_SDCFG_DDR2_TERM0);
		setup->setParam->ddr2Term1 = CSL_FEXT(tmpReg, DDR2_SDCFG_DDR2_TERM1);
		setup->setParam->ddrDisDll = CSL_FEXT(tmpReg, DDR2_SDCFG_DDR_DISABLE_DLL);
		setup->setParam->sdramDrive0 = (CSL_Ddr2SdramDrStrength)CSL_FEXT(tmpReg, DDR2_SDCFG_SDRAM_DRIVE0);
    }
    
    /* Get the SDRAM Timing1 Configuration */
    if(setup->timing1Param != NULL) {
        tmpReg = hDdr2->regs->SDTIM1;
        setup->timing1Param->trfc = CSL_FEXT(tmpReg, DDR2_SDTIM1_T_RFC);
        setup->timing1Param->trp = CSL_FEXT(tmpReg, DDR2_SDTIM1_T_RP);
        setup->timing1Param->trcd = CSL_FEXT(tmpReg, DDR2_SDTIM1_T_RCD);
        setup->timing1Param->twr = CSL_FEXT(tmpReg, DDR2_SDTIM1_T_WR);
        setup->timing1Param->tras = CSL_FEXT(tmpReg, DDR2_SDTIM1_T_RAS);
        setup->timing1Param->trc = CSL_FEXT(tmpReg, DDR2_SDTIM1_T_RC);
        setup->timing1Param->trrd = CSL_FEXT(tmpReg, DDR2_SDTIM1_T_RRD);
        setup->timing1Param->twtr = CSL_FEXT(tmpReg, DDR2_SDTIM1_T_WTR);
    }
    
    /* Get the SDRAM Timing2 Configuration */
    if (setup->timing2Param != NULL) {
        tmpReg = hDdr2->regs->SDTIM2;
        setup->timing2Param->todt = CSL_FEXT(tmpReg, DDR2_SDTIM2_T_ODT);
        setup->timing2Param->tsxnr = CSL_FEXT(tmpReg, DDR2_SDTIM2_T_XSNR);
        setup->timing2Param->tsxrd = CSL_FEXT(tmpReg, DDR2_SDTIM2_T_XSRD);
        setup->timing2Param->trtp = CSL_FEXT(tmpReg, DDR2_SDTIM2_T_RTP);
        setup->timing2Param->tcke = CSL_FEXT(tmpReg, DDR2_SDTIM2_T_CKE);
    }
    return status;
}

