/*  ============================================================================
 *   Copyright (c) Texas Instruments Inc 2002, 2003, 2004, 2005, 2008
 *
 *   Use of this software is controlled by the terms and conditions found
 *   in the license agreement under which this software has been supplied.
 *  ============================================================================
 */

/** 
 *  @file    csl_ddr2HwSetup.c
 *
 *  @brief   File for functional layer of CSL API @a CSL_ddr2HwSetup()
 *
 *  @path:   $(CSLPATH)\src\ddr2
 *
 *  @desc
 *    - The @a CSL_ddr2HwSetup() function definition & it's associated
 *      functions
 *
 *  Modification 1
 *    - Modified on: 11/4/2005
 *    - Reason: created the sources
 *
 *  @date 11th April, 2005
 *  @author Ramitha Mathew.
 *
 */ 

/*  ============================================================================
 *  Revision History
 *  ================
 *  30-May-2008 Updated SDCFG register bit field names according to UG.
 *  26-Mar-2008 Lock the BOOT_UNLOCK bit to prevent from making further changes.
 *  08-Sep-2008 Update for Doxygen
 *  11-Nov-2008 Added fields ddr2Ddqs, ddr2Term0, ddr2Term1, ddrDisDll, sdramDrive0. 
 *              Updated DDR2_SDTIM2_T_XSNR and DDR2_SDTIM2_T_XSRD field according to UG
 *  ============================================================================
 */

#include <csl_ddr2.h>


#pragma CODE_SECTION (CSL_ddr2HwSetup, ".text:csl_section:ddr2");
CSL_Status  CSL_ddr2HwSetup (
    CSL_Ddr2Handle     hDdr2,
    CSL_Ddr2HwSetup    *setup
)
{
    CSL_Status status = CSL_SOK;
    Uint32 tmpReg = 0;
    
    /* invalid parameter checking */
    if (setup == NULL) {
        return CSL_ESYS_INVPARAMS;
    }
    

    /* bad handle checking*/
    if (hDdr2 == NULL) {
        return CSL_ESYS_BADHANDLE;
    }    
    /* Unlock boot and timing fields */
    tmpReg = hDdr2->regs->SDCFG;
    CSL_FINST(tmpReg, DDR2_SDCFG_BOOT_UNLOCK, UNLOCKED);
	CSL_FINST(tmpReg, DDR2_SDCFG_TIMUNLOCK, SET);
	hDdr2->regs->SDCFG = tmpReg;
	
    /* Setup the SDRAM Timing1 Register */
    /* All fields which are writable with timing_unlock set are edited since the
     *  unlock bit is set above
     */
    if(setup->timing1Param != NULL) {
        tmpReg = hDdr2->regs->SDTIM1;
        CSL_FINS(tmpReg, DDR2_SDTIM1_T_RFC, setup->timing1Param->trfc);
        CSL_FINS(tmpReg, DDR2_SDTIM1_T_RP, setup->timing1Param->trp);
        CSL_FINS(tmpReg, DDR2_SDTIM1_T_RCD, setup->timing1Param->trcd);
        CSL_FINS(tmpReg, DDR2_SDTIM1_T_WR, setup->timing1Param->twr);
        CSL_FINS(tmpReg, DDR2_SDTIM1_T_RAS, setup->timing1Param->tras);
        CSL_FINS(tmpReg, DDR2_SDTIM1_T_RC, setup->timing1Param->trc);
        CSL_FINS(tmpReg, DDR2_SDTIM1_T_RRD, setup->timing1Param->trrd);
        CSL_FINS(tmpReg, DDR2_SDTIM1_T_WTR, setup->timing1Param->twtr);       
        hDdr2->regs->SDTIM1 = tmpReg;
    }
    
    /* Setup the SDRAM Timing2 Register */
    /* All fields which are writable with timing_unlock set are edited since the
     *  unlock bit is set above
     */
    if (setup->timing2Param != NULL) {
        tmpReg = hDdr2->regs->SDTIM2;
        CSL_FINS(tmpReg, DDR2_SDTIM2_T_ODT, setup->timing2Param->todt);
        CSL_FINS(tmpReg, DDR2_SDTIM2_T_XSNR, setup->timing2Param->tsxnr);
        CSL_FINS(tmpReg, DDR2_SDTIM2_T_XSRD, setup->timing2Param->tsxrd);
        CSL_FINS(tmpReg, DDR2_SDTIM2_T_RTP, setup->timing2Param->trtp);
        CSL_FINS(tmpReg, DDR2_SDTIM2_T_CKE, setup->timing2Param->tcke);
        hDdr2->regs->SDTIM2 = tmpReg;
    }

	/* Set PHY Read latency */
    CSL_FINS(hDdr2->regs->DMCCTL, DDR2_DMCCTL_RL, setup->readLatncy);

    /* Refresh rate*/     
    CSL_FINS(hDdr2->regs->SDRFC, DDR2_SDRFC_REFRESH_RATE, setup->refreshRate);  

    
    /* Writing the ddr2 sdram Settings in SDRAM Config register */  
    if (setup->setParam != NULL) {                   
        tmpReg = hDdr2->regs->SDCFG;
        CSL_FINS(tmpReg, DDR2_SDCFG_CL, setup->setParam->casLatncy);
        CSL_FINS(tmpReg, DDR2_SDCFG_IBANK, setup->setParam->ibank);
        CSL_FINS(tmpReg, DDR2_SDCFG_NM, setup->setParam->narrowMode);
        CSL_FINS(tmpReg, DDR2_SDCFG_PAGESIZE, setup->setParam->pageSize);
        CSL_FINS(tmpReg, DDR2_SDCFG_SDRAM_ENABLE, setup->setParam->sdramEn);
        CSL_FINS(tmpReg, DDR2_SDCFG_DDR2_ENABLE, setup->setParam->ddr2En);
        CSL_FINS(tmpReg, DDR2_SDCFG_DDR_ENABLE, setup->setParam->ddrEn);
        CSL_FINS(tmpReg, DDR2_SDCFG_DDR2_DDQS, setup->setParam->ddr2Ddqs);
        CSL_FINS(tmpReg, DDR2_SDCFG_DDR2_TERM0, setup->setParam->ddr2Term0);
        CSL_FINS(tmpReg, DDR2_SDCFG_DDR2_TERM1, setup->setParam->ddr2Term1);
        CSL_FINS(tmpReg, DDR2_SDCFG_DDR_DISABLE_DLL, setup->setParam->ddrDisDll);
        CSL_FINS(tmpReg, DDR2_SDCFG_SDRAM_DRIVE0, setup->setParam->sdramDrive0);
        /* Lock the BOOT_UNLOCK bit to prevent from making further changes */
		hDdr2->regs->SDCFG = (tmpReg & (~(CSL_DDR2_SDCFG_BOOT_UNLOCK_MASK)));
    }
      
    /* Locking the timing_unlock to prevent further changes */
    CSL_FINST(hDdr2->regs->SDCFG, DDR2_SDCFG_TIMUNLOCK, CLEAR);        

    return status;
}

